#!/usr/bin/env python
# -*- coding: utf-8 -*-

from libavg import avg, logger
from Tile import Tile
from World import World

class CheckpointManager():

    # enum of kinds of checkpoints
    CHECKPOINT_REGULAR = 0
    CHECKPOINT_INTERSECTION = 1
    CHECKPOINT_START = 2

    def __init__(self, maxCheckpointLength, parentDiv=None):
        self.__recentPlayerPositions = [] # Content: Tuple (TilePos, orientation, kind)
        self.__maxCheckpointLength = maxCheckpointLength
        self.__mainDiv = avg.DivNode()

        if parentDiv is not None:
            if not isinstance(parentDiv, avg.DivNode):
                logger.error("Can not use 'drawhelpers' without valid avg.divNode parent")
            else:
                self.__parentDiv = parentDiv
                self.__mainDiv = avg.DivNode(parent=self.__parentDiv)
                self.__cpHelper = avg.RectNode(size=Tile.getTileSize(), strokewidth=1, color="7D26CD", sensitive=False, parent=self.__mainDiv) # purple

    def __del__(self):
        del self.__recentPlayerPositions
        del self.__maxCheckpointLength
        self.destroy()

    def show(self, show):
        if self.__mainDiv:
            self.__mainDiv.active = show

    def destroy(self):
        if self.__mainDiv:
            self.__mainDiv.unlink(True)
            self.__mainDiv = None


    # Adds player positions and orientations (Checkpoints) according to the following conditions:
    #   * The player can maximally be reset the number of tiles defined in 'self.__maxCheckpointLength'
    #   * The player will be resetted to the last intersection if the distance is less than the maximum defined above
    def addCheckpoint(self, tilePos, orientation, kind):
        if kind == CheckpointManager.CHECKPOINT_INTERSECTION:
            self.__recentPlayerPositions = [(tilePos, orientation, kind)]
        elif len(self.__recentPlayerPositions) >= self.__maxCheckpointLength:
            del self.__recentPlayerPositions[0]
            self.__recentPlayerPositions.append((tilePos, orientation, kind))
        else:
            self.__recentPlayerPositions.append((tilePos, orientation, kind))

    def getLastValidCheckpoint(self, reset=False):
        if reset:
            self.resetCheckpoints()
        return self.__recentPlayerPositions[0]

    def resetCheckpoints(self):
        del self.__recentPlayerPositions[1:]

    def drawHelpers(self):
        self.__mainDiv.pos = World.getPixelPos(self.getLastValidCheckpoint()[0])
