#!/usr/bin/env python
# -*- coding: utf-8 -*-

import math

# Rotates an orthogonal vector by 90 degrees clockwise
# (x,y) - One of these tuples: [(1,0),(-1,0),(0,1),(0,-1)]
def rotateVec90CW((x,y)):
    return {
        (1, 0): (0, 1),
        (0, 1): (-1, 0),
        (-1, 0): (0, -1),
        (0, -1): (1, 0)
    }[(x,y)]

# Rotates list1 clockwise about 90 until it is a subset of list2
# list1, list2 - Lists of direction vectors (tuples)
# Returns:
#   - the rotated angle if a match was found (may be 0)
#   - 0 if there is no match possible
def matchVectors(list1, list2):
    if len(list1) < len(list2):
        return 0

    angle = 0
    while angle < 360:
        matches = True
        for d1 in list1:
            if not d1 in list2:
                matches = False
                break

        if matches:
            return angle
        else:
            list1 = [rotateVec90CW(d1) for d1 in list1]
            angle += 90

    return 0


class Direction:
    UP = None
    DOWN = None
    LEFT = None
    RIGHT = None

    def __init__(self, val=0):
        self.val = val

    def isValid(self):
        return self.val in xrange(4)

    def __eq__(self, other):
        if isinstance(other, Direction):
            return other.val == self.val
        elif type(other) is int:
            return other == self.val
        elif type(other) is str:
            return other.lower() == `self`.lower()
        else:
            return False

    def __str__(self):
        if self.isValid():
            return ['Up', 'Right', 'Down', 'Left'][self.val]
        else:
            return '?Direction(val=' + `self.val` + ')?'

    __repr__ = __str__


    # (x,y) - One of [(1,0),(-1,0),(0,1),(0,-1)]
    # If (x,y) is invalid, this function returns None
    @staticmethod
    def fromTuple((x,y)):
        dirs = {
            (1,0): Direction.RIGHT,
            (-1,0): Direction.LEFT,
            (0,1): Direction.DOWN,
            (0,-1): Direction.UP
        }

        if (x,y) in dirs:
            return dirs[(x,y)]
        else:
            return None


    # Returns a tile-position tuple that represents a movement into this direction
    def tuple(self):
        if self.isValid():
            return [(0,-1), (1,0), (0,1), (-1,0)][self.val]
        else:
            return (0,0)

    # Calculates the clockwise angle from this to the other Direction in degrees
    # If other is not given, it defaults to Direction.UP
    # Returns 0 if this or the other Direction is invalid
    def angle(self, other=None):
        if not other:
            other = Direction.UP

        if self.isValid() and other.isValid():
            otherVal = other.val
            if otherVal < self.val:
                otherVal += 4
            return (otherVal - self.val) * 90
        else:
            return 0

    # Returns the clockwise angle from this to the other Direction in radians
    # If other is not given, it defaults to Direction.UP
    # Returns 0 if this or the other Direction is invalid
    def angleRadians(self, other=None):
        return math.radians(self.angle(other))

    # Returns the inverted direction as a new Direction, i.e.
    # the direction vector rotated by 180 degrees
    def inverted(self):
        return Direction((self.val + 2) % 4)

    # Returns this direction rotated by 90 degrees clockwise.
    # This is faster than converting it to a tuple, rotating and converting it back
    def rotatedBy90CW(self):
        return Direction((self.val + 1) % 4)

# Do not change the order, it has to be ordered clockwise!
Direction.UP = Direction(0)
Direction.RIGHT = Direction(1)
Direction.DOWN = Direction(2)
Direction.LEFT = Direction(3)
