# coding=utf-8

import os.path
import json
from libavg import logger

class HighScore():

    HIGHSCORE_FILE_PATH = "../highscore.json"
    MAX_ENTRIES = 25 # How many values can be saved at maximum

    # Serializes Score and writes it to Highscore File
    # Returns HighScore placement [1 to MAX_ENTRIES] or None if not an highscore
    @staticmethod
    def publishScore(teamName, score):

        highscoreData = HighScore.getHighscoreEntries()

        numEntries = len(highscoreData)

        ID = HighScore.getPossibleHighscorePlacement(score, highscoreData)
        if ID is None:
            logger.info("Current score too low, not adding to highscore.")
            return None

        # Increase id of all succeeding scores, remove last entry
        numEntries = min(numEntries, HighScore.MAX_ENTRIES - 1)
        for i in xrange(numEntries, ID, -1): # dict index starts at 0 => highest valid value is HighScore.MAX_ENTRIES - 1
            highscoreData[str(i)] = highscoreData[str(i-1)]

        # Add new entry
        entry = {"team": teamName, "score": score}
        highscoreData[str(ID)] = entry

        # Update file
        with open(HighScore.HIGHSCORE_FILE_PATH, "w+") as foo:
            foo.write(json.dumps(highscoreData, separators=[',',':']))

        return ID + 1

    # Check if a score is high enough to be added to the highscore
    # Returns HighScore placement [0 to MAX_ENTRIES-1] or None if not an highscore
    @staticmethod
    def getPossibleHighscorePlacement(score, highscoreData=None):
        if highscoreData is None:
            highscoreData = HighScore.getHighscoreEntries()

        numEntries = len(highscoreData)
        if numEntries is 0:
            ID = 0
        else:
            ID = HighScore.MAX_ENTRIES
            for i in xrange(numEntries - 1, -1, -1):
                if score > highscoreData[str(i)]["score"]:
                    ID = i + 1
                    break
                ID = i

        if ID >= HighScore.MAX_ENTRIES:
            return None

        return ID

    # returns highscore data as dict:
    # {"id": {"team": "teamName", "score": scoreValue}}
    @staticmethod
    def getHighscoreEntries():
        highscoreData = {} # return empty dict if file not existent
        if os.path.isfile(HighScore.HIGHSCORE_FILE_PATH): # when there is a file, we assume it is syntactically corect

            with open(HighScore.HIGHSCORE_FILE_PATH, "r") as foo:
                data = foo.read()
            try:
                highscoreData = json.loads(data)
            except ValueError, e:
                highscoreData = {}

        return highscoreData


    # Returns pango formatted string that can easyly be represented by an avg.WordsNode
    @staticmethod
    def getHumanReadableHighscoreEntries():
        highscoreData = HighScore.getHighscoreEntries()
        numEntries = len(highscoreData)

        highScoreString = ""

        # sort by id and thus by best score
        currentID = 0
        while True:
            if currentID >= numEntries:
                break
            for scoreID in highscoreData:
                if scoreID == str(currentID):
                    scoreMinutes = int(round(highscoreData[scoreID]["score"] * 0.001 / 60.0))
                    scoreSeconds = int(round((highscoreData[scoreID]["score"] * 0.001) % 60.0))
                    scoreText = ""
                    if scoreMinutes > 0:
                        scoreText += `scoreMinutes` + " Minute" + ("n" if scoreMinutes > 1 else "") + " "
                    if scoreSeconds > 0:
                        scoreText += `scoreSeconds` + " Sekunde" + ("n" if scoreSeconds > 1 else "")
                    if scoreMinutes == 0 and scoreSeconds == 0:
                        scoreText += "Hacker??"
                    
                    highScoreString += "<b><big>" + str(currentID + 1) + ":</big></b> " + highscoreData[scoreID]["team"] + " : " + scoreText + "<br/>"
                    currentID += 1
                    break

        return highScoreString
