from libavg import avg, player as avgPlayer
from World import *
from Tile import *
from Direction import *

class Player(avg.DivNode):
    # (x,y) - tile position of the player
    def __init__(self, (x,y), orientation, assetHref, parentDiv):
        super(Player, self).__init__(pos=World.getPixelPos((x,y)), pivot=0.5 * avg.Point2D(Tile.getTileSize()), sensitive=False)

        self.registerInstance(self, parentDiv)

        orientationTuple = orientation.tuple()
        self.__targetTilePos = (x,y)
        self.angle = Player.getAngleFromOrientation(orientation)

        self.__orientation = orientation
        self.__lastOrientation = orientation # Backup of the previous orientation after setting a new target pos

        self.__playerImgDiv = avg.DivNode(size=Tile.getTileSize(), parent=self)
        tileSz = Tile.getTileSize()
        playerNode = self.__playerImgDiv
        if playerNode.width < playerNode.height:
            playerNode.height *= tileSz[0] / playerNode.width
            playerNode.width = tileSz[0]
        else:
            playerNode.width *= tileSz[1] / playerNode.height
            playerNode.height = tileSz[1]

        self.__playerImg = avg.ImageNode(size=self.__playerImgDiv.size, href=assetHref, parent=self.__playerImgDiv)

        self.balance = 0
        self.tilt = 0.0     # player tilt due to curves
        self.tiltDir = 0    # direction, the tilt will change
        self.speed = 0.0    # px per sec

        if avgPlayer.drawHelpers:
            self.__targetTileHelper = avg.RectNode(pos=self.targetPos, size=Tile.getTileSize(), strokewidth=1, color="00FF00", sensitive=False, parent=parentDiv) # green
            self.__destTileHelper = avg.RectNode(pos=self.pos, size=Tile.getTileSize(), strokewidth=1, color="0000FF", sensitive=False, parent=parentDiv) # blue
            self.__remainHelper = avg.LineNode(pos1=self.pos, pos2=self.targetPos, strokewidth=3, color="00FFFF", sensitive=False, parent=parentDiv) # turqouise
            self.__playerAnchor = avg.CircleNode(r=2, color="000000", strokewidth=2, parent=self)


    # Returns the player tile position as a tuple
    # This method takes into account the numerical problems occuring during the
    # movement of the player. Thus, it rounds the position UP, in case it exceeded
    # a given threshold.
    def getTilePos(self, threshold=0.98):
        tileSz = avg.Point2D(Tile.getTileSize())
        pos = avg.Point2D(self.pos) / tileSz
        frac = pos % 1.0
        rounded = avg.Point2D(float(frac.x > threshold), float(frac.y > threshold))
        tilePos = pos - frac + rounded
        return (int(tilePos.x), int(tilePos.y))

    tilePos = property(getTilePos)


    # Returns the tile-position as a tuple that the player is targeting
    def getTargetTilePos(self):
        return self.__targetTilePos

    def setTargetTilePos(self, targetTilePos):
        self.__targetTilePos = targetTilePos

        tilePos = self.getTilePos(0.94)

        self.__lastOrientation = self.__orientation
        self.__orientation = Direction.fromTuple((self.__targetTilePos[0] - tilePos[0], self.__targetTilePos[1] - tilePos[1]))
        if self.__orientation is None:
            tileSz = avg.Point2D(Tile.getTileSize())
            pos = avg.Point2D(self.pos) / tileSz
            frac = pos % 1.0
            print "Orientation is None!! frac = " + `frac`

    targetTilePos = property(getTargetTilePos, setTargetTilePos)


    # Returns the pixel-position as a Point2D that the player is targeting
    def getTargetPos(self):
        return avg.Point2D(self.__targetTilePos) * avg.Point2D(Tile.getTileSize())

    targetPos = property(getTargetPos)


    # Returns the tile-position of the player when he started to move towards the
    # target position, as a tuple. I.e. the tile-position of the tile that the player
    # already reached.
    def getOriginTilePos(self):
        dirTuple = self.__orientation.tuple()
        return (self.__targetTilePos[0] - dirTuple[0], self.__targetTilePos[1] - dirTuple[1])

    originTilePos = property(getOriginTilePos)


    # Returns the orientation of the player as a Direction
    def getOrientation(self):
        return self.__orientation

    orientation = property(getOrientation)


    def getLastOrientation(self):
        return self.__lastOrientation

    # Resets the last orientation to the current orientation
    # Use with care!! - thus not added to the property!
    def resetLastOrientation(self):
        self.__lastOrientation = self.orientation

    lastOrientation = property(getLastOrientation)


    # Returns the player angle in radians representing the given orientation
    @staticmethod
    def getAngleFromOrientation(orientation):
        return Direction.UP.angleRadians(orientation) # asset faces north


    def getImageDiv(self):
        return self.__playerImgDiv

    imageDiv = property(getImageDiv)


    def getImageNode(self):
        return self.__playerImg

    imageNode = property(getImageNode)


    def onFrame(self, dt):
        if avgPlayer.drawHelpers:
            self.__remainHelper.pos1 = self.pos
            self.__remainHelper.pos2 = self.targetPos
            self.__targetTileHelper.pos = self.targetPos
            self.__destTileHelper.pos = self.tilePos
