  # coding=utf-8

class StatisticsManager():

    # Enum - All valid statistics kinds
    T_MC = 1 # Specifies time spend in the minecart / identifier specifies Level ID
    T_FSE = 2 # Specifies time in FullscreenEvents / identifier specifies FSE
    C_CRASH = 3 # Count of Crashs / identifier specifies kind of crash
    C_FSE = 4 # How many FSE events have been started
    C_ITEMS_COLLECTED = 5 # How many items have been collected
    C_ITEMS_FULLY_USED = 6 # How many items have completele been used

    STATISTICS = {
        T_MC: {"desc": "Zeit im Minecart", "unit": "ms"}
        , T_FSE: {"desc": "Zeit in Full Screen Events", "unit": "ms"}
        , C_CRASH: {"desc": "Anzahl der Crashes", "unit": "count"}
        , C_FSE: {"desc": "Anzahl der Minispiele", "unit": "count"}
        , C_ITEMS_COLLECTED: {"desc": "Anzahl eingesammelter Items", "unit": "count"}
        , C_ITEMS_FULLY_USED: {"desc": "Anzahl vollständig aufgebrauchter Items", "unit": "count"}
    }

    def __init__(self):
        self.reset()

    def reset(self):
        self.__statistics = {}
        for k in StatisticsManager.STATISTICS:
            self.__statistics[k] = {}

    def addValue(self, kind, identifier, value):
        # abort if non valid value
        if kind not in StatisticsManager.STATISTICS:
            return
        if identifier in self.__statistics[kind]:
            self.__statistics[kind][identifier] += value
        else:
            self.__statistics[kind][identifier] = value

    def getValues(self, kind):
        if kind not in StatisticsManager.STATISTICS:
            return None
        return self.__statistics[kind]

    def getCompleteTime(self):
        time = 0
        for timeType in [StatisticsManager.T_MC, StatisticsManager.T_FSE]:
            for specificTime in self.__statistics[timeType]:
                time += self.__statistics[timeType][specificTime]
        return time

    def getStatistics(self):
        return self.__statistics

    # Returns pango formatted string containing all statistics that can easyly be represented by an avg.WordsNode
    def getHumanReadableStatistics(self):
        stats = self.__statistics
        statsString = ""

        for kind in stats:
            statsString += "○ <b>" + StatisticsManager.STATISTICS[kind]["desc"] + ":</b><br/>"

            preIdentifierString = ""
            if kind == StatisticsManager.T_MC:
                preIdentifierString = "Level "

            for identifier in stats[kind]:

                if StatisticsManager.STATISTICS[kind]["unit"] == "ms":
                    valueString = str(round(stats[kind][identifier] * 0.001, 1)) + " Sekunden"
                else:
                    valueString = str(int(stats[kind][identifier]))

                statsString += "\t• " + preIdentifierString + identifier + ": " + valueString + "<br/>"
            statsString += "<br/>"

        return statsString
