from libavg import avg
import sys

# min1, max1, min2, max2 - libavg.avg.Point2D's
def intersectAABB(min1, max1, min2, max2):
    return (min1.x <= max2.x) and (min1.y <= max2.y) and (max1.x >= min2.x) and (max1.y >= min2.y)

# Returns the AABB (min & max Point2Ds) for the given area node	as a tuple
def getAreaNodeAABB(areaNode):
    return areaNode.pos, areaNode.pos + areaNode.size

# Helper class describing an Axis Aligned Bound Box
class AABB:
    def __init__(self, min=None, max=None):
        self.min = min
        if self.min is None:
            self.min = avg.Point2D(sys.float_info.max, sys.float_info.max)
        self.max = max
        if self.max is None:
            self.max = avg.Point2D(-sys.float_info.max, -sys.float_info.max)

    def reset(self):
        self.min = avg.Point2D(sys.float_info.max, sys.float_info.max)
        self.max = avg.Point2D(-sys.float_info.max, -sys.float_info.max)

    # Include given Point2D
    def add(self, point):
        if point.x < self.min.x:
            self.min.x = point.x
        if point.x > self.max.x:
            self.max.x = point.x
        if point.y < self.min.y:
            self.min.y = point.y
        if point.y > self.max.y:
            self.max.y = point.y

    # Returns True if this AABB contains the given Point2D point, False if not
    def contains(self, point):
        return (self.min.x <= point.x <= self.max.x) and (self.min.y <= point.y <= self.max.y)

    # Returns True if this AABB intersects the other AABB
    def intersects(self, other):
        return (self.min.x <= other.max.x) and (self.min.y <= other.max.y) and (self.max.x >= other.min.x) and (self.max.y >= other.min.y)

    # vec - Point2D
    def outset(self, vec):
        self.min -= vec
        self.max += vec
        return self

    # vec - Point2D
    def inset(self, vec):
        self.min += vec
        self.max -= vec
        return self

    def __str__(self):
        return "AABB(min=" + `self.min` + ", max=" + `self.max` + ")"


# Multiplies either two Point2Ds or a Point2Ds and a scalar
def mulPoint2D(a, b):
    if isinstance(b, (int, long, float)):
        return avg.Point2D(a.x * b, a.y * b)
    else:
        return avg.Point2D(a.x * b.x, a.y * b.y)

# Divides either two Point2Ds or a Point2D by a scalar
def divPoint2D(a, b):
    if isinstance(b, (int, long, float)):
        binv = 1.0 / b
        return avg.Point2D(a.x * binv, a.y * binv)
    else:
        return avg.Point2D(a.x / b.x, a.y / b.y)

# Takes the modulo of either two Point2Ds or a Point2D an a scalar
def modPoint2D(a, b):
    if isinstance(b, (int, long, float)):
        return avg.Point2D(a.x % b, a.y % b)
    else:
        return avg.Point2D(a.x % b.x, a.y % b.y)

# Dot product between two avg.Point2D's returning a scalar
def dotPoint2D(a, b):
    return a.x * b.x + a.y * b.y

# Return the first child node that is an instance of the supplied class, else None
def getFirstChildNodeofClass(node, Class):
    for i in xrange(node.getNumChildren()):
        child = node.getChild(i)
        if child.__class__ == Class:
            return child
    return None

avg.Point2D.__mul__ = mulPoint2D
avg.Point2D.__div__ = divPoint2D
avg.Point2D.__mod__ = modPoint2D
avg.Point2D.dot = dotPoint2D


# Returns the given tuple as a {x,y} dict
def serializeVector((x,y)):
    return { 'x': x, 'y': y }

# Tries to convert the given {x,y} dict into a tuple.
# The data may also be a list [x,y].
# Returns None if the given dict was not valid.
def deserializeVector(data):
    if isinstance(data, dict) and ('x' in data and 'y' in data):
        return (data['x'],data['y'])
    elif isinstance(data, list) and len(data) >= 2:
        return (list[0],list[1])
    else:
        return None


# Makes sure the given node is rendered above each other child of the node's parent
def bringNodeToTop(node):
    if node and node.parent:
        node.parent.reorderChild(node, node.parent.getNumChildren() - 1)
