from libavg import avg, player as avgplayer
from TileEvent import *
import math
import random

class LockpickEvent(TileEvent):
    EVENT_NAME = "lockpick_event"

    def __init__(self, tile):
        super(LockpickEvent, self).__init__(tile)
        self.__lockedImage = avg.ImageNode(href="../assets/lockpick/door_closed.png", pos=tile.pos, size=tile.size, angle=math.radians(tile.rotation), parent=tile.parent)

        self.__completed = False

    def getEventName(self):
        return LockpickEvent.EVENT_NAME

    def isFullscreenEvent(self):
        return True

    def onFrame(self, dt, keystates):
        if self.__completed:
            return True
        else:
            return False

    def onInit(self, parentDiv):
        self.__parent = parentDiv
        self.__direction = avg.Point2D(0,0)

        self.__CtrlContact1 = None
        self.__CtrlContact2 = None
        self.__CtrlContact3 = None
        self.__CtrlContact4 = None

        offsetTop = 60
        offsetSide = 90
        circleD = 250
        self.__circleR = circleD * 0.25

        self.__background = avg.ImageNode(href="../assets/lockpick/bg.png", size=parentDiv.size, parent=parentDiv)

        # Setup 4 Divs for the Circle Controls
        self.__circleDiv1 = avg.DivNode(pos=(parentDiv.width - circleD - offsetSide, parentDiv.height - circleD - offsetTop), size=(circleD, circleD), parent=parentDiv)
        self.__circleDiv1.subscribe(avg.Node.CURSOR_MOTION, lambda e: self.__onCtrlCursorMotion(e,0))
        parentDiv.subscribe(avg.Node.CURSOR_UP, lambda e: self.__onCtrlCursorUp(e,0))

        self.__circleDiv2 = avg.DivNode(pos=(parentDiv.width - circleD - offsetSide, 0 + offsetTop), size=(circleD, circleD), parent=parentDiv)
        self.__circleDiv2.subscribe(avg.Node.CURSOR_MOTION, lambda e: self.__onCtrlCursorMotion(e,1))
        parentDiv.subscribe(avg.Node.CURSOR_UP, lambda e: self.__onCtrlCursorUp(e,1))

        self.__circleDiv3 = avg.DivNode(pos=(0 + offsetSide, 0 + offsetTop), size=(circleD, circleD), parent=parentDiv)
        self.__circleDiv3.subscribe(avg.Node.CURSOR_MOTION, lambda e: self.__onCtrlCursorMotion(e,2))
        parentDiv.subscribe(avg.Node.CURSOR_UP, lambda e: self.__onCtrlCursorUp(e,2))

        self.__circleDiv4 = avg.DivNode(pos=(0 + offsetSide, parentDiv.height - circleD - offsetTop), size=(circleD, circleD), parent=parentDiv)
        self.__circleDiv4.subscribe(avg.Node.CURSOR_MOTION, lambda e: self.__onCtrlCursorMotion(e,3))
        parentDiv.subscribe(avg.Node.CURSOR_UP, lambda e: self.__onCtrlCursorUp(e,3))

        # Create 4 control circles
        circleRadius = 90
        ctrlCircleRadius = 30
        self.__ctrlCircleCenter1 = avg.CircleNode(pos=self.__circleDiv1.size*0.5, color="353535", strokewidth=2, opacity=1, fillcolor="505050",r=ctrlCircleRadius, fillopacity=1, parent=self.__circleDiv1)
        self.__ctrlCircle1 = avg.CircleNode(pos=self.__ctrlCircleCenter1.pos, color="353535", strokewidth=2, opacity=1, fillcolor="505050",r=circleRadius, fillopacity=0.7, parent=self.__circleDiv1)

        self.__ctrlCircleCenter2 = avg.CircleNode(pos=self.__circleDiv2.size*0.5, color="353535", strokewidth=2, opacity=1, fillcolor="505050",r=ctrlCircleRadius, fillopacity=1, parent=self.__circleDiv2)
        self.__ctrlCircle2 = avg.CircleNode(pos=self.__ctrlCircleCenter2.pos, color="353535", strokewidth=2, opacity=1, fillcolor="505050",r=circleRadius, fillopacity=0.7, parent=self.__circleDiv2)

        self.__ctrlCircleCenter3 = avg.CircleNode(pos=self.__circleDiv3.size*0.5, color="353535", strokewidth=2, opacity=1, fillcolor="505050",r=ctrlCircleRadius, fillopacity=1, parent=self.__circleDiv3)
        self.__ctrlCircle3 = avg.CircleNode(pos=self.__ctrlCircleCenter3.pos, color="353535", strokewidth=2, opacity=1, fillcolor="505050",r=circleRadius, fillopacity=0.7, parent=self.__circleDiv3)

        self.__ctrlCircleCenter4 = avg.CircleNode(pos=self.__circleDiv4.size*0.5, color="353535", strokewidth=2, opacity=1, fillcolor="505050",r=ctrlCircleRadius, fillopacity=1, parent=self.__circleDiv4)
        self.__ctrlCircle4 = avg.CircleNode(pos=self.__ctrlCircleCenter4.pos, color="353535", strokewidth=2, opacity=1, fillcolor="505050",r=circleRadius, fillopacity=0.7, parent=self.__circleDiv4)

        # Calculate offset
        self.__ctrlCircleOffset1 = self.__circleDiv1.size * 0.5 + self.__circleDiv1.pos
        self.__ctrlCircleOffset2 = self.__circleDiv2.size * 0.5 + self.__circleDiv2.pos
        self.__ctrlCircleOffset3 = self.__circleDiv3.size * 0.5 + self.__circleDiv3.pos
        self.__ctrlCircleOffset4 = self.__circleDiv4.size * 0.5 + self.__circleDiv4.pos

        # Subscribe to register player interaction with controls
        self.__ctrlCircle1.subscribe(avg.Node.CURSOR_DOWN, lambda e: self.__onCtrlCursorDown(e,0))
        self.__ctrlCircle2.subscribe(avg.Node.CURSOR_DOWN, lambda e: self.__onCtrlCursorDown(e,1))
        self.__ctrlCircle3.subscribe(avg.Node.CURSOR_DOWN, lambda e: self.__onCtrlCursorDown(e,2))
        self.__ctrlCircle4.subscribe(avg.Node.CURSOR_DOWN, lambda e: self.__onCtrlCursorDown(e,3))

        # Setup "keyhole" the players have to click
        self.__keyhole = avg.CircleNode(pos=parentDiv.size*0.5 + (0,50), color="353535", strokewidth=2, opacity=1, fillcolor="505050",r=20, fillopacity=1, parent=parentDiv)

        # Randomize where the Plates get placed
        offsetX = random.randint(10,40)
        offsetY = random.randint(10,40)
        self.__offset1 = (offsetX,offsetY)
        self.__offset2 = (-offsetX,offsetY)
        self.__offset3 = (offsetX,-offsetY)
        self.__offset4 = (-offsetX,-offsetY)

        plateSizeMult = 0.8

        # Choose a color
        color = random.choice(['yellow', 'red', 'green', 'blue'])
        assets = {
            'down': 'coin_' + color + '_down.png',
            'up': 'coin_' + color + '_up.png',
            'left': 'coin_' + color + '_left.png',
            'right': 'coin_' + color + '_right.png'
        }

        # Place invisible base Plates used for correctly moving around the real plates
        self.__plateBottom = avg.ImageNode(href="../assets/lockpick/" + assets['down'], size=(466*plateSizeMult,194*plateSizeMult), pos=parentDiv.size*0.5 - (235,-70), parent=parentDiv, opacity=0.0, sensitive=False)
        self.__plateTop = avg.ImageNode(href="../assets/lockpick/" + assets['up'], size=(466*plateSizeMult,197*plateSizeMult), pos=parentDiv.size*0.5 - (235,140), parent=parentDiv, opacity=0.0, sensitive=False)
        self.__plateLeft = avg.ImageNode(href="../assets/lockpick/" + assets['left'], size=(196*plateSizeMult,78*plateSizeMult), pos=parentDiv.size*0.5 - (235,-15), parent=parentDiv, opacity=0.0, sensitive=False)
        self.__plateRight = avg.ImageNode(href="../assets/lockpick/" + assets['right'], size=(195*plateSizeMult,78*plateSizeMult), pos=parentDiv.size*0.5 - (17,-15), parent=parentDiv, opacity=0.0, sensitive=False)

        # Place the Plates the players will be using with their corresponding offset
        self.__plateBottom1 = avg.ImageNode(href=self.__plateBottom.href, size=self.__plateBottom.size, pos=self.__plateBottom.pos + self.__offset1, opacity=1.0 , parent=parentDiv)
        self.__plateTop1 = avg.ImageNode(href=self.__plateTop.href, size=self.__plateTop.size, pos=self.__plateTop.pos + self.__offset1, opacity=1.0 , parent=parentDiv)
        self.__plateLeft1 = avg.ImageNode(href=self.__plateLeft.href, size=self.__plateLeft.size, pos=self.__plateLeft.pos + self.__offset1, opacity=1.0 , parent=parentDiv)
        self.__plateRight1 = avg.ImageNode(href=self.__plateRight.href, size=self.__plateRight.size, pos=self.__plateRight.pos + self.__offset1, opacity=1.0, parent=parentDiv)

        self.__plateBottom2 = avg.ImageNode(href=self.__plateBottom.href, size=self.__plateBottom.size, pos=self.__plateBottom.pos + self.__offset2, opacity=1.0 , parent=parentDiv)
        self.__plateTop2 = avg.ImageNode(href=self.__plateTop.href, size=self.__plateTop.size, pos=self.__plateTop.pos + self.__offset2, opacity=1.0 , parent=parentDiv)
        self.__plateLeft2 = avg.ImageNode(href=self.__plateLeft.href, size=self.__plateLeft.size, pos=self.__plateLeft.pos + self.__offset2, opacity=1.0 , parent=parentDiv)
        self.__plateRight2 = avg.ImageNode(href=self.__plateRight.href, size=self.__plateRight.size, pos=self.__plateRight.pos + self.__offset2, opacity=1.0, parent=parentDiv)

        self.__plateBottom3 = avg.ImageNode(href=self.__plateBottom.href, size=self.__plateBottom.size, pos=self.__plateBottom.pos + self.__offset3, opacity=1.0 , parent=parentDiv)
        self.__plateTop3 = avg.ImageNode(href=self.__plateTop.href, size=self.__plateTop.size, pos=self.__plateTop.pos + self.__offset3, opacity=1.0 , parent=parentDiv)
        self.__plateLeft3 = avg.ImageNode(href=self.__plateLeft.href, size=self.__plateLeft.size, pos=self.__plateLeft.pos + self.__offset3, opacity=1.0 , parent=parentDiv)
        self.__plateRight3 = avg.ImageNode(href=self.__plateRight.href, size=self.__plateRight.size, pos=self.__plateRight.pos + self.__offset3, opacity=1.0, parent=parentDiv)

        self.__plateBottom4 = avg.ImageNode(href=self.__plateBottom.href, size=self.__plateBottom.size, pos=self.__plateBottom.pos + self.__offset4, opacity=1.0 , parent=parentDiv)
        self.__plateTop4 = avg.ImageNode(href=self.__plateTop.href, size=self.__plateTop.size, pos=self.__plateTop.pos + self.__offset4, opacity=1.0 , parent=parentDiv)
        self.__plateLeft4 = avg.ImageNode(href=self.__plateLeft.href, size=self.__plateLeft.size, pos=self.__plateLeft.pos + self.__offset4, opacity=1.0 , parent=parentDiv)
        self.__plateRight4 = avg.ImageNode(href=self.__plateRight.href, size=self.__plateRight.size, pos=self.__plateRight.pos + self.__offset4, opacity=1.0, parent=parentDiv)

        # Lists to correctly move around the Circles and Plates
        self.__contacts = [self.__CtrlContact1, self.__CtrlContact2, self.__CtrlContact3, self.__CtrlContact4]
        self.__circleCenters = [self.__ctrlCircleCenter1, self.__ctrlCircleCenter2, self.__ctrlCircleCenter3, self.__ctrlCircleCenter4]
        self.__circles = [self.__ctrlCircle1, self.__ctrlCircle2, self.__ctrlCircle3, self.__ctrlCircle4]
        self.__circleOffsets = [self.__ctrlCircleOffset1,self.__ctrlCircleOffset2, self.__ctrlCircleOffset3,self.__ctrlCircleOffset4]
        self.__plateBottomsPossible = [self.__plateBottom1, self.__plateBottom2, self.__plateBottom3, self.__plateBottom4]
        self.__plateTopsPossible = [self.__plateTop1, self.__plateTop2, self.__plateTop3, self.__plateTop4]
        self.__plateLeftsPossible = [self.__plateLeft1, self.__plateLeft2, self.__plateLeft3, self.__plateLeft4]
        self.__plateRightsPossible = [self.__plateRight1, self.__plateRight2, self.__plateRight3, self.__plateRight4]
        self.__offsetsPossible = [self.__offset1,self.__offset2, self.__offset3, self.__offset4]

        # Randomize the player order
        self.__players = [0,1,2,3]
        random.shuffle(self.__players)

        # Set up the real Lists
        self.__plateBottoms = []
        self.__plateTops = []
        self.__plateLefts = []
        self.__plateRights = []
        self.__offsets = []

        # Populate real lists according to random order
        for x in self.__players:
            self.__plateBottoms.append(self.__plateBottomsPossible[x])
            self.__plateTops.append(self.__plateTopsPossible[x])
            self.__plateLefts.append(self.__plateLeftsPossible[x])
            self.__plateRights.append(self.__plateRightsPossible[x])
            self.__offsets.append(self.__offsetsPossible[x])

        # Subscribe to keyhole that needs to be pressed
        self.__keyhole.subscribe(avg.Node.CURSOR_DOWN, self.__complete)
        avgplayer.subscribe(avgplayer.KEY_UP, self.__onKeyUp)

    def __onCtrlCursorUp(self, event, num):
        if self.__contacts[num] == event.contact:
            self.__contacts[num] = None

    def __onCtrlCursorDown(self, event, num):
        if self.__contacts[num] == None:
            self.__contacts[num] = event.contact

    def __onCtrlCursorMotion(self, event, num):
        if (self.__contacts[num] != None) and (event.contact == self.__contacts[num]):
            a = event.pos - self.__circleOffsets[num]
            radius = a.getNorm()
            if radius > self.__circleR:
                radius = self.__circleR
            a = (avg.Point2D(0,0) if radius < 0.0001 else a.getNormalized() * radius)
            self.__direction = a
            # Move circle around
            self.__circleCenters[num].pos = self.__circles[num].pos + self.__direction

            # Move Plates around
            directionFactor = 1.7
            self.__plateBottoms[num].pos = self.__plateBottom.pos + self.__offsets[num] + self.__direction * directionFactor
            self.__plateTops[num].pos = self.__plateTop.pos + self.__offsets[num] + self.__direction * directionFactor
            self.__plateLefts[num].pos = self.__plateLeft.pos + self.__offsets[num] + self.__direction * directionFactor
            self.__plateRights[num].pos = self.__plateRight.pos + self.__offsets[num] + self.__direction * directionFactor

    def __complete(self, event):
        self.__completed = True

    def __onKeyUp(self, event):
        if event.keyname == 'S':
            self.__complete(event)

    def onDisabled(self):
        self.__lockedImage.href="../assets/lockpick/door_open.png"
        self.__lockedImage = None
