from libavg import avg, player as avgPlayer
from random import randint
from TileEvent import TileEvent
import math


class MemoryEvent(TileEvent):
    EVENT_NAME="memory_event"

    def getEventName(self):
        return self.EVENT_NAME

    def isFullscreenEvent(self):
        return True

    #Rounds = How many you match in a row
    def __init__(self, tile, rounds=5):
        super(MemoryEvent, self).__init__(tile)
        self.rounds = rounds
        self.__door = avg.ImageNode(pos=tile.pos, href="../assets/door.png", angle=math.radians(tile.rotation), size=tile.getTileSize(), parent=tile.parent)
        self.__door.sensitive = False
        self.__completed = False

    def onInit(self, parentDiv):

        self.__globalDiv = parentDiv

        #Scaling Stuff
        gameSize = avg.Point2D(900,900) #Default Size due to Size of Background

        gameOffset = (parentDiv.size - gameSize)* 0.5 #Scale to use for centering the minigame

        #Scale used for
        scale = parentDiv.size[0]/900

        if parentDiv.size[1]/(900)<scale:
            scale = parentDiv.size[1]/(900)


        #background and Buttons
        self.__background = avg.ImageNode(pos=avg.Point2D(0,0)+gameOffset, href="../assets/memory_game/background.png", parent=parentDiv)
        self.__icon1 = avg.ImageNode(pos=avg.Point2D(410,90)+gameOffset,href="../assets/memory_game/icon1.png", parent=parentDiv)
        self.__icon2 = avg.ImageNode(pos=avg.Point2D(100,320)+gameOffset,href="../assets/memory_game/icon2.png", parent=parentDiv)
        self.__icon3 = avg.ImageNode(pos=avg.Point2D(740,320)+gameOffset,href="../assets/memory_game/icon3.png", parent=parentDiv)
        self.__icon4 = avg.ImageNode(pos=avg.Point2D(610,683)+gameOffset,href="../assets/memory_game/icon4.png", parent=parentDiv)
        self.__icon5 = avg.ImageNode(pos=avg.Point2D(230,687)+gameOffset,href="../assets/memory_game/icon5.png", parent=parentDiv)
        self.__error = avg.ImageNode(pos=avg.Point2D(200,200)+gameOffset, href="../assets/memory_game/error.png", parent=parentDiv)
        self.__error.opacity = 0.0

        #Add Divs for Click detection
        div1 = avg.DivNode(pos=avg.Point2D(350,40)+gameOffset, size=(150*scale,150*scale), parent=parentDiv)
        div2 = avg.DivNode(pos=avg.Point2D(40,265)+gameOffset, size=(150*scale,150*scale), parent=parentDiv)
        div3 = avg.DivNode(pos=avg.Point2D(670,265)+gameOffset, size=(150*scale,150*scale), parent=parentDiv)
        div4 = avg.DivNode(pos=avg.Point2D(550,635)+gameOffset, size=(150*scale,150*scale), parent=parentDiv)
        div5 = avg.DivNode(pos=avg.Point2D(165,635)+gameOffset, size=(150*scale,150*scale), parent=parentDiv)


        #Subscribe to divs
        div1.subscribe(avg.Node.CURSOR_DOWN, lambda e, n=0: self.press(e,n))
        div2.subscribe(div2.CURSOR_DOWN, lambda e, n=1: self.press(e,n))
        div3.subscribe(div3.CURSOR_DOWN, lambda e, n=2: self.press(e,n))
        div4.subscribe(div4.CURSOR_DOWN, lambda e, n=3: self.press(e,n))
        div5.subscribe(div5.CURSOR_DOWN, lambda e, n=4: self.press(e,n))

        #Create Lifes
        self.__life1 = avg.ImageNode(size=(60,60), pos=avg.Point2D(860,860)+gameOffset, href="../assets/memory_game/heart.png", parent=parentDiv)
        self.__life2 = avg.ImageNode(size=(60,60), pos=avg.Point2D(830,860)+gameOffset, href="../assets/memory_game/heart.png", parent=parentDiv)
        self.__life3 = avg.ImageNode(size=(60,60), pos=avg.Point2D(800,860)+gameOffset, href="../assets/memory_game/heart.png", parent=parentDiv)

        #Initialize Variables
        self.__lifeList = [self.__life1, self.__life2, self.__life3]
        self.__iconList = [self.__icon1,self.__icon2,self.__icon3,self.__icon4,self.__icon5]
        self.__userInput = []
        self.__currentTest = 0
        self.__tries = 3
        self.__status = False
        self.__showRoundBool = False
        self.__showRoundBool2 = False
        self.__currentFadeIn = 0
        self.__currentFadeOut = 0
        self.__player = avg.Player.get()
        self.__setupComplete = False
        self.__blocked = False

        #Set Game up
        self.setupRound()

        self.__timer = self.__player.setTimeout(1000, lambda n=0: self.showRoundNumber(0))
        #Enable to see what you need to press
        #print(self.__solution)
        avgPlayer.subscribe(avgPlayer.KEY_UP, self.__onKeyUp)


    #Create the Random Answer array that has to be reproduced
    def setupRound(self):
        self.__solution = []

        for x in range(0,self.rounds):
            self.__solution.append(randint(0,4))
            

    def showRoundNumber(self, number):
        currentNode = self.__iconList[self.__solution[number]]
        avg.Anim.fadeOut(currentNode,400, lambda n=number: self.showRoundNumberEnd(n))

    def showRoundNumberEnd(self, number):
        currentNode = self.__iconList[self.__solution[number]]
        number += 1
        if number == len(self.__solution):
            avg.Anim.fadeIn(currentNode,400,1.0)
            self.__setupComplete = True
        else:
            avg.Anim.fadeIn(currentNode,1000,1.0, lambda n=number: self.showRoundNumber(n))

    #Checks if the last Input is true and if the game has been solved
    def solveRound(self):
        self.__blocked = True
        self.__inputAnimation = self.__iconList[self.__userInput[len(self.__userInput)-1]]
        self.__showInput = avg.Anim.fadeIn(self.__inputAnimation,100,0.5,self.showInput)
        if self.__tries > 0:
            if  self.__solution[self.__currentTest] == self.__userInput[self.__currentTest]:
                self.__currentTest += 1
                self.__blocked = False
                if self.__solution == self.__userInput:
                    self.__completed = True
            else:
                self.__currentTest = 0
                self.__userInput = []
                self.deductLife()


    #Fade back in background for user input
    def showInput(self):
        self.__showInput = avg.Anim.fadeIn(self.__inputAnimation,100,1.0)

    #Used for the click events
    def press(self, event, number):
        if self.__setupComplete and not self.__blocked:
            self.__userInput.append(number)
            self.solveRound()

    #Resets the Game
    def gameOver(self):
        self.onInit(self.__globalDiv)

    #Fades out the lifes when they are deducted
    def deductLife(self):
        avg.Anim.fadeIn(self.__lifeList[self.__tries-1],250,0.0)
        self.__tries -= 1
        self.__showError()

    def __showError(self):
        avg.Anim.fadeIn(self.__error, 1000, 1.0, self.__resetError)

    def __resetError(self):
        self.__error.opacity = 0.0
        if self.__tries == 0:
            self.gameOver()
        self.__blocked = False

    #Required for Integration into the Maingame
    def onFrame(self, dt, keystates):
        if self.__completed:
            return True

    #Easy Solve for Debugging
    def __onKeyUp(self, event):
        if (event.keyname == 'S'):
            self.__completed = True

    def onDisabled(self):
        self.__door.href="../assets/door_open.png"
        self.__door = None
