from libavg import avg
from TileEvent import *
import math
import random

class Portal(TileFeature):
    FEATURE_NAME = "portals_feature"
    MIN_PORTAL_DISTANCE = 5

    # otherPos - Tile-position tuple of the other portal or None to place randomly
    def __init__(self, tile, otherPos=None, isSecondary=False, hueNode=None):
        super(Portal, self).__init__(tile)

        hueNode = (avg.HueSatFXNode(hue=random.randint(-180,180)) if not hueNode else hueNode)

        worldNode = tile.parent
        self.__portalImg = avg.ImageNode(
            href="../assets/portal.png",
            pos=tile.pos,
            size=tile.size,
            angle=math.radians(tile.rotation),
            parent=worldNode)
        self.__portalImg.setEffect(hueNode)

        world = tile.getWorld()

        self.__isSecondary = isSecondary
        if isSecondary:
            self.__otherPos = otherPos
        else:
            if otherPos is not None:
                otherTile = world.getTile(otherPos)

            if otherPos is None or otherTile is None:
                # Find fitting, completely random position in world with
                # at least a little distance from each other
                mazePath = world.getMazePath()
                pos = avg.Point2D(tile.position)
                otherTile = None
                for i in xrange(len(mazePath) * 2):
                    checkTilePos = random.choice(mazePath)
                    checkTile = world.getTile(checkTilePos)
                    distance = (avg.Point2D(checkTilePos) - pos).getNorm()
                    if distance >= Portal.MIN_PORTAL_DISTANCE:
                        continue
                    elif not checkTile.texture == "path_straight" or checkTile.hasFeature(TileEvent.FEATURE_NAME):
                        continue
                    elif world.isCrossing(checkTilePos) or world.isNextToCrossing(checkTilePos):
                        continue
                    else:
                        otherTile = checkTile
                        break

                # In the rare case that we found no fitting tile, just use a fallback tile.
                if otherTile is None:
                    otherTile = mazePath[0]

            self.__otherPos = otherTile.position

            # Create the second portal
            otherFeature = Portal(otherTile, tile.position, True, hueNode)
            otherTile.addFeature(otherFeature)

    def getFeatureName(self):
        return Portal.FEATURE_NAME

    # Returns the tile-position-tuple of the portal that corresponds to this one
    def getOtherPortalPosition(self):
        return self.__otherPos
