from libavg import avg, player as avgPlayer
from random import *
from TileEvent import TileEvent
import math


class QuicktimeEvent(TileEvent):
    EVENT_NAME = "quicktime_event"
    BLINK_TIME = 300

    def getEventName(self):
        return self.EVENT_NAME

    def isFullscreenEvent(self):
        return True

    def __init__(self, tile):
        super(QuicktimeEvent, self).__init__(tile)
        self.__tileImage = avg.ImageNode(pos=tile.pos, href="../assets/door.png", angle=math.radians(tile.rotation), size=tile.getTileSize(), parent=tile.parent)
        self.__tileImage.sensitive = False
        self.__kindaCompleted = False
        self.__completed = False

    def onInit(self, parentDiv):
        self.__globalDiv = parentDiv

        divCenter = parentDiv.size * 0.5

        self.__solutionCircle1 = avg.CircleNode(pos=(divCenter[0] - 400, divCenter[1]), r=50, fillcolor="ffffff", fillopacity=0.0, strokewidth=2, opacity=0.0, parent=parentDiv)
        self.__solutionCircle2 = avg.CircleNode(pos=(divCenter[0] - 200, divCenter[1]), r=50, fillcolor="ffffff", fillopacity=0.0, strokewidth=2, opacity=0.0, parent=parentDiv)
        self.__solutionCircle3 = avg.CircleNode(pos=(divCenter[0] + 10, divCenter[1]), r=50, fillcolor="ffffff", fillopacity=0.0, strokewidth=2, opacity=0.0, parent=parentDiv)
        self.__solutionCircle4 = avg.CircleNode(pos=(divCenter[0] + 210, divCenter[1]), r=50, fillcolor="ffffff", fillopacity=0.0, strokewidth=2, opacity=0.0, parent=parentDiv)
        self.__solutionCircle5 = avg.CircleNode(pos=(divCenter[0] + 420, divCenter[1]), r=50, fillcolor="ffffff", fillopacity=0.0, strokewidth=2, opacity=0.0, parent=parentDiv)
        self.__solutionCircles = [self.__solutionCircle1, self.__solutionCircle2, self.__solutionCircle3, self.__solutionCircle4, self.__solutionCircle5]


        avgPlayer.subscribe(avgPlayer.KEY_UP, self.__onKeyUp)
        self.__playerColors = ["df0000", "00df00", "0000df", "df9100"]
        shuffle(self.__playerColors)

        self.__player0 = avg.CircleNode(pos=(300, 300), r=100, fillcolor=self.__playerColors[0], fillopacity=0.0, strokewidth=4, opacity=0.0, parent=parentDiv)
        self.__player1 = avg.CircleNode(pos=(300, parentDiv.size[1]-300), r=100, fillcolor=self.__playerColors[1], fillopacity=0.0, strokewidth=4, opacity=0.0, parent=parentDiv)
        self.__player2 = avg.CircleNode(pos=(parentDiv.size[0]-300, 300), r=100, fillcolor=self.__playerColors[2], fillopacity=0.0, strokewidth=4, opacity=0.0, parent=parentDiv)
        self.__player3 = avg.CircleNode(pos=(parentDiv.size[0]-300, parentDiv.size[1]-300), r=100, fillcolor=self.__playerColors[3], fillopacity=0.0, strokewidth=4, opacity=0.0, parent=parentDiv)

        self.__players = [self.__player0, self.__player1, self.__player2, self.__player3]
        for i in xrange(len(self.__players)):
            self.__players[i].originalColor = self.__playerColors[i]


        self.__failureNode = avg.RectNode(pos=(50,50), size=parentDiv.size - avg.Point2D(100,100), fillcolor="ff0000", fillopacity=0.0, sensitive=False, strokewidth=0, parent=parentDiv)
        self.__failureAnim = avg.LinearAnim(node=self.__failureNode, attrName="fillopacity", startValue=1.0, endValue=0.0, duration=1000, stopCallback=self.__activateButtons)

        self.__background = avg.ImageNode(size=parentDiv.size, href="../assets/color-game/bg.png", sensitive=False, parent=parentDiv)

        self.__player0.subscribe(self.__player0.CURSOR_DOWN, lambda e, n=0: self.__press(e, n))
        self.__player1.subscribe(self.__player0.CURSOR_DOWN, lambda e, n=1: self.__press(e, n))
        self.__player2.subscribe(self.__player0.CURSOR_DOWN, lambda e, n=2: self.__press(e, n))
        self.__player3.subscribe(self.__player0.CURSOR_DOWN, lambda e, n=3: self.__press(e, n))

        self.__blockDiv = avg.DivNode(size=parentDiv.size, pos=(0, 0), parent=parentDiv)

        self.__setupRound()

        avgPlayer.getRootNode().subscribe(avg.Node.CURSOR_UP, self.__onCursorUp)


    def __setupRound(self):
        for x in range (0,len(self.__solutionCircles)):
            self.__solutionCircles[x].r = 70
            self.__solutionCircles[x].opacity = 0.0
        self.__blockDiv.active = True
        self.__setupSolution()
        avg.Player.get().setTimeout(1000, lambda n=0: self.__showSolution(0))

    def __setupSolution(self):
        self.__solution = []
        self.__pressed = []
        self.__current = 0
        self.__tries = 3

        for x in range(0,5):
            self.__solution.append(randint(0,3))

    def __showSolution(self, count):
        if count < len(self.__solution):
            self.__solutionCircles[count].fillcolor = self.__playerColors[self.__solution[count]]
            anim = avg.LinearAnim(self.__solutionCircles[count], "fillopacity", self.BLINK_TIME, 0.0, 1.0, stopCallback=lambda n=count: self.__hideSolution(n))
            anim.start()

    def __hideSolution(self, count):
        anim = avg.LinearAnim(self.__solutionCircles[count], "fillopacity", self.BLINK_TIME, 1.0, 0.0, stopCallback=lambda n=count+1: self.__showSolution(n))
        anim.start()
        if count+1 == len(self.__solution):
            for x in range (0,len(self.__solutionCircles)):
                self.__solutionCircles[x].opacity = 1.0
            self.__blockDiv.active = False
            self.__showPlayerCircles()

    def __press(self, event, count):
        event.node.fillcolor = "e4e4e4"
        avg.Anim.fadeIn(self.__players[count], 300, 1.0, lambda n=count: self.__showButton(n))
        self.__pressed.append(count)
        if self.__solution[self.__current] == self.__pressed[self.__current]:
            self.__solutionCircles[self.__current].fillopacity = 1.0
            self.__current += 1
            if self.__solution == self.__pressed:
                self.__onCompleted()
        else:
            self.__resetCenterCircles()
            self.__current = 0
            self.__pressed = []
            self.__wrongInput()

    def __onCursorUp(self, event):
        if self.__kindaCompleted:
            return

        for player in self.__players:
            player.fillcolor = player.originalColor

    def __showButton(self, count):
        self.__players[count].opacity = 0.0

    def __resetCenterCircles(self):
        for x in range(0,len(self.__solutionCircles)):
            self.__solutionCircles[x].fillopacity = 0.0

    def __wrongInput(self):
        self.__tries -= 1
        if self.__tries == 0:
            self.__blockDiv.active = True
            self.__failureAnim.setStopCallback(self.__setupRound)
            self.__failureAnim.start()
            return

        self.__background.sensitive = True
        self.__failureAnim.setStopCallback(self.__activateButtons)
        self.__failureAnim.start()

    def __activateButtons(self):
        self.__background.sensitive = False

    def __showPlayerCircles(self):
        for x in range(0, len(self.__players)):
            self.__players[x].fillopacity = 1.0

    #Required for Integration into the Maingame
    def onFrame(self, dt, keystates):
        if self.__completed:
            return True

    #Easy Solve for Debugging
    def __onKeyUp(self, event):
        if (event.keyname == 'S'):
            self.__onCompleted()

    def __onCompleted(self):
        self.__kindaCompleted = True
        self.__blockDiv.active = True
        self.__blockDiv.sensitive = True

        green = "00df00"
        for player in self.__players:
            player.fillcolor = green
        for solutionCircle in self.__solutionCircles:
            solutionCircle.fillcolor = green

        avgPlayer.setTimeout(2000, self.__onReallyCompleted)

    def __onReallyCompleted(self):
        self.__completed = True

    def onDisabled(self):
        self.__tileImage.href="../assets/door_open.png"
        self.__tileImage = None
