from libavg import avg
from Tile import *
from TileEvent import *
from random import choice
from math import radians

class SpearTrap(TileFeature):
    FEATURE_NAME = "speartrap_feature"
    NUM_DESTRUCTION_TAPS = 7 # number of taps required to destroy the SpearTrap

    # leverPos - Tile-position-tuple of the lever or None to place randomly
    def __init__(self, tile, leverPos=None):
        super(SpearTrap, self).__init__(tile)
        self.__activated = True

        worldRootNode = tile.parent
        self.__trapImg = avg.ImageNode(
            href="../assets/spear-trap/spear-trap-active.png",
            pos=tile.pos,
            size=tile.size,
            parent=worldRootNode)

        self.__trapImg.angle = radians(tile.getRotation())

        self.__trapImg.subscribe(avg.Node.CURSOR_DOWN, lambda e: self.__onCursorDown())
        self.__damage = 0

        world = tile.getWorld()

        if leverPos is not None:
            leverTile = world.getTile(leverPos)

        if leverPos is None or leverTile is None:
            # Add lever to disable the trap somewhere around the trap in a limited range
            # If no free place was found for the lever, the range will be increased next try
            nearbyPathTiles = []
            maxNearbyDist = 0
            while len(nearbyPathTiles) == 0:
                maxNearbyDist += 8 # Try with 5 more blocks in each direction
                for y in xrange(tile.position[1] - maxNearbyDist, tile.position[1] + maxNearbyDist):
                    for x in xrange(tile.position[0] - maxNearbyDist, tile.position[0] + maxNearbyDist):
                        if (x,y) != tile.position and world.isPathTile((x,y)) and not world.isCrossing((x,y)):
                            nearbyTile = world.getTile((x,y))
                            if nearbyTile.getTexture() == "path_straight" and not nearbyTile.hasFeature(TileEvent.FEATURE_NAME):
                                nearbyPathTiles.append((x,y))
            leverPos = choice(nearbyPathTiles)
            leverTile = world.getTile(leverPos)

        self.__lever = leverTile.getFeature(SpearTrapLever.FEATURE_NAME)
        if not self.__lever:
            self.__lever = SpearTrapLever(leverTile)
            leverTile.addFeature(self.__lever)

        self.__lever.registerTrap(self)

    def getFeatureName(self):
        return SpearTrap.FEATURE_NAME

    # Returns the SpearTrapLever Feature that disables this SpearTrap
    def getLever(self):
        return self.__lever

    def __onCursorDown(self):
        if self.__activated:
            self.__damage += 1
            if self.__damage >= SpearTrap.NUM_DESTRUCTION_TAPS:
                self.__trapImg.href = "../assets/spear-trap/spear-trap-destroyed.png"
                self.__activated = False
                if self.__lever:
                    self.__lever.deactivate()

    def onPlayerEnteredTile(self, **kwargs):
        if self.__activated:
            self.__trapImg.href = "../assets/spear-trap/spear-trap-active-triggered.png"

    def onPlayerLeftTile(self):
        if self.__activated:
            self.__trapImg.href = "../assets/spear-trap/spear-trap-active.png"

    def isActivated(self):
        return self.__activated

    def deactivate(self):
        self.__activated = False
        self.__trapImg.href = "../assets/spear-trap/spear-trap-deactive.png"


class SpearTrapLever(TileFeature):
    FEATURE_NAME = "speartrap_lever_feature"

    def __init__(self, tile):
        super(SpearTrapLever, self).__init__(tile)
        self.__traps = [] # list of traps handled by this lever

        worldRootNode = tile.parent

        self.__leverImg = avg.ImageNode(
            href="../assets/spear-trap/lever-active.png",
            pos=tile.pos,
            angle=math.radians(tile.rotation),
            parent=worldRootNode)

        self.__leverImg.subscribe(avg.Node.CURSOR_DOWN, self.__onCursorDown)

    def getFeatureName(self):
        return SpearTrapLever.FEATURE_NAME

    def onPlayerEnteredTile(self, **kwargs):
        self.__onDeactivate()

    def __onCursorDown(self, event):
        self.__onDeactivate()

    def __onDeactivate(self):
        for trap in self.__traps:
            trap.deactivate()
        self.deactivate()

    def registerTrap(self, trap):
        if trap and not trap in self.__traps:
            self.__traps.append(trap)

    def deactivate(self):
        self.__leverImg.href = "../assets/spear-trap/lever-deactive.png"
