from abc import ABCMeta, abstractmethod
from Tile import *

class TileEvent(TileFeature):
    __metaclass__ = ABCMeta
    FEATURE_NAME = "tileevent_feature"

    def __init__(self, tile):
        super(TileEvent, self).__init__(tile)
        self.__disabled = False

    def getFeatureName(self):
        return TileEvent.FEATURE_NAME

    # Returns the name of the event that is bound to this tile
    @abstractmethod
    def getEventName(self):
        return "unknown_event"

    # Returns True if this TileEvent is a fullscreen event that
    # can be started with onInit().
    @abstractmethod
    def isFullscreenEvent(self):
        return False

    # Returns True if the TileEvent requires a countdown before
    # the game resumes after completing the event.
    def requiresCountdownAfterCompletion(self):
        return True

    # Returns True if this event was disabled / solved
    def isDisabled(self):
        return self.__disabled

    # Mark this event as solved / disabled
    def disable(self):
        self.__disabled = True
        self.onDisabled()

    def enable(self):
        self.__disabled = False
        self.onEnabled()

    # Callback called when this tile event was solved / disabled
    # Overwrite this method in your TileEvent implementation if you need to
    def onDisabled(self):
        pass

    def onEnabled(self):
        pass

    # Called to initialize and start the FULLSCREEN event
    def onInit(self, parentDiv):
        pass

    # Called every frame if this is a fullscreen event.
    # This method returns True if the fullscreen event is
    # completed, otherwise it returns False.
    def onFrame(self, dt, keystats):
        return True
