  # coding=utf-8

from libavg import avg
from event.TileEvent import *
from menu.ScreenMessenger import ScreenMessenger
from menu.StyleDefinitions import StyleDefinitions

class TutorialEvent(TileEvent):
    EVENT_NAME = "tutorial_event"
    MAX_MESSAGE_COUNT = 8 # No infinite cycles, sorry

    ASSET_DIR = "../assets/tutorial/"
    TUTORIALS = {
        0 : {"audio": ASSET_DIR + "tut_00_welcome.ogg", "head": "Willkommen im Tutorial!", "desc": "Im Folgenden könnt ihr euch mit den grundlegenden Interaktionsmöglichkeiten vertraut machen. <br/> <br/> Nach dem Video könnt ihr euch an der Eingabe direkt ausprobieren. <br/> <br/> Drückt auf 'Weiter', wenn ihr bereit seid.", "displayNext":1}
        ,1 : {"href": ASSET_DIR + "tut_01_switches.mp4", "audio": ASSET_DIR + "tut_01_switches.ogg", "head": "Weichen stellen", "desc": "Ihr wollt alternative Routen erkunden?<br/>Kein Problem - drückt auf einen der Kreuzungseingänge und zieht den Finger bis zum gewünschten Ende der Kreuzung, um die Weichen zu stellen."}
        ,2 : {"href": ASSET_DIR + "tut_02_breaks.mp4", "audio": ASSET_DIR + "tut_02_breaks.ogg", "head": "Bremsen", "desc": "Im Laufe des Spiels werdet ihr nicht umhinkommen, die Geschwindigkeit zu reduzieren, um schwierige Situationen zu meistern.<br/><br/>Zieht dazu den Hebel zum Rand des Bildschirms.<br/> Passt jedoch auf eure Bremsen nicht zu überhitzen!"}
        ,3 : {"href": ASSET_DIR + "tut_03_balance.mp4", "audio": ASSET_DIR + "tut_03_balance.ogg", "head": "Balance", "desc": "Auf alten Schienen fährt es sich sehr ungemütlich. Damit ihr nicht umkippt, muss ein Spieler die Balance halten.<br/><br/>Nutzt dafür die Balance-Steuerung in der Bildschirmecke durch tippen auf den Bereich links oder rechts neben der Lore."}
        ,4 : {"href": ASSET_DIR + "tut_04_light.mp4", "audio": ASSET_DIR + "tut_04_light.ogg", "head": "Licht", "desc": "Selbstverständlich ist es im Dungeon komplett finster. Das macht jedoch nichts, da ihr eine Lampe dabei habt.<br/><br/>Um Licht ins Dunkle zu bringen, müsst ihr auf den inneren Kreis der Lampe drücken, und ihn anschließend in die gewünschte Richtung ziehen."}
        ,5 : {"href": ASSET_DIR + "tut_05_compass.mp4", "audio": ASSET_DIR + "tut_05_compass.ogg", "head": "Karte und Kompass", "desc": "Ihr seid nun mit der grundlegenden Steuerung vertraut, fragt euch aber, wohin eure Reise gehen soll?<br/><br/>In einer Bildschirmecke gibt es genau für diesen Zweck eine Karte mit einem magischen Kompass, der in Richtung der Schatztruhe zeigt."}
        ,6 : {"href": ASSET_DIR + "tut_06_obstacles.mp4", "audio": ASSET_DIR + "tut_06_obstacles.ogg", "head": "Hindernisse", "desc": "Hin und wieder werden Hindernisse oder Fallen euren Weg versperren. Tippt mehrmals auf die Hindernisse um sie zu zerstören und findet den geheimen Schalter, um die Fallen zu deaktivieren. Andererseits werden auch Fallen zu euren Hindernissen...<br/><br/>An einigen Stellen sind die Schienen beschädigt. Fahrt hier unbedingt zur anderen Seite geneigt vorbei!"}
    }

    def __init__(self, tile, tutID):
        super(TutorialEvent, self).__init__(tile)
        self.__id = tutID
        self.__messageCount = 0

    def getEventName(self):
        return TutorialEvent.EVENT_NAME

    def isFullscreenEvent(self):
        return True

    def requiresCountdownAfterCompletion(self):
        return False

    def onInit(self, parentDiv):
        self.__messageCount += 1
        self.__parent = parentDiv

        self.__mainDiv = avg.DivNode(size=parentDiv.size, parent=parentDiv)
        self.__mainBG = avg.RectNode(size=self.__mainDiv.size, color="000000", fillcolor="000000", fillopacity=1.0, parent=self.__mainDiv)
        self.__bgImage = avg.RectNode(size=self.__mainDiv.size, parent=self.__mainDiv, filltexhref=StyleDefinitions.MENU_BG_IMAGE, fillopacity=1, opacity=0)
        tutorial = TutorialEvent.TUTORIALS[self.__id]

        marginX = 52 # horizontal margin between elements

        # default message layout without video
        vidRelWidth = 0.0 # Percentage of diplay width dedicated to video size (not including  horizontal margins)
        msgPos = (0,0)
        msgSz = (self.__mainDiv.size.x * (1.0 - vidRelWidth),self.__mainDiv.size.y)
        msgMargin = True

        self.__tutVideo = None
        if "href" in TutorialEvent.TUTORIALS[self.__id]:
            href = tutorial["href"]
            vidRelWidth = 0.7
            vidSz = self.__mainDiv.size * vidRelWidth
            self.__tutVideo = avg.VideoNode(href=href, size=vidSz, loop=True, parent=parentDiv) # Generate Video Prototype to get aspect ratio
            vidXNew = self.__tutVideo.size.x - marginX * 3 # Remove margin space and set new video width
            self.__tutVideo.size = (vidXNew, (vidXNew / self.__tutVideo.size.x) * self.__tutVideo.size.y) # scale video and preserve aspect ratio
            self.__tutVideo.pos = (self.__mainDiv.size.x - self.__tutVideo.size.x - marginX, self.__mainDiv.size.y * 0.5 - 0.5 * self.__tutVideo.size.y)
            self.__tutVideo.play()

            # Resize and reposition message if video is present
            msgSz = (self.__mainDiv.size.x * (1.0 - vidRelWidth), self.__tutVideo.size.y)
            msgPos = (marginX, self.__tutVideo.pos.y)
            msgMargin = False

        self.__tutAudio = None
        if "audio" in TutorialEvent.TUTORIALS[self.__id]:
            self.__tutAudio = avg.SoundNode(href=tutorial["audio"], parent=parentDiv)
            self.__tutAudio.play()

            if self.__id is not 0:
                self.__replayButton = avg.ImageNode(href=TutorialEvent.ASSET_DIR +"restart.png", pos=self.__tutVideo.pos - (100,-self.__tutVideo.size[1]+75), size=(50,50), parent=parentDiv)
                self.__replayButton.subscribe(avg.Node.CURSOR_DOWN, lambda e: self.__resetAudio())

        self.__message = ScreenMessenger(size=msgSz, pos=msgPos, headline=tutorial["head"], message=tutorial["desc"], margin=msgMargin, drawBG=False, parent=self.__mainDiv)



    def onFrame(self, dt, keystates):
        if self.__message.isFinished():
            return self.__onFinish()

    def onDestroy(self):
        self.__parent = None

    def __onFinish(self):
        tutorial = TutorialEvent.TUTORIALS[self.__id]
        self.__reset()

        # If another Tutorial should be displayed immediately
        # Delete old notes and add new ones
        if "displayNext" in tutorial and self.__messageCount < TutorialEvent.MAX_MESSAGE_COUNT:
            self.__id = tutorial["displayNext"]
            self.onInit(self.__parent)

        # If there are no further tutorials attached, return from this event
        else:
            return True

    def __resetAudio(self):
        if self.__tutAudio:
            self.__tutAudio.stop()
            self.__tutAudio.play()

    def __reset(self):
        if self.__tutVideo:
            self.__tutVideo.unlink(True)
            self.__tutVideo = None
        if self.__message:
            self.__message.unlink(True)
            self.__message = None
        if self.__tutAudio:
            self.__tutAudio.unlink(True)
            self.__tutAudio = None
        self.__mainDiv.unlink(True)
        self.__mainDiv = None
