from libavg import avg, logger, geom, player as avgPlayer
from Tile import *
from math import radians

class BalanceControls:

    def __init__(self, parentDiv, player):
        self.__player = player

        self.active = True
        player.balance = 0.0 # positive -> right, negative -> left

        # Controls HUD
        cSz = avg.Point2D(250, 250) # controlsSize - square
        cBM = 5 # ControlButtonMargin
        cBtSz = avg.Point2D(cSz.x * 0.5 - cBM, cSz.y) # controlButtonSize
        imgSz = cSz * 0.75 # player image size
        self.__mainDiv = avg.DivNode(pos=(parentDiv.width - cSz.x, 25), size=cSz + (0, (parentDiv.height / 12)), parent=parentDiv)
        self.__controlsMainDiv = avg.DivNode(pos=(0, 0), size=cSz, pivot=cSz * 0.5, angle=radians(180), parent=self.__mainDiv) # Controlable HUD element in top right corner
        self.__controlImg = avg.ImageNode(href="../assets/HUD/balance_middle.png", pos=(cSz * 0.5 - imgSz * 0.5), sensitive=False, size=imgSz, parent=self.__controlsMainDiv)

        self.__controlShadow = avg.ImageNode(href="../assets/HUD/balance_shadow.png", opacity=1.0, parent=self.__controlsMainDiv)

        self.__controlSwitchDiv = avg.DivNode(pos =(self.__mainDiv.width - 60, self.__mainDiv.height - 70), size=(35, 35), parent=self.__mainDiv)
        self.__switchStatus = geom.PieSlice(radius=(self.__controlSwitchDiv.width * 0.5) + 5, startangle=0, endangle=0, pos=(self.__controlSwitchDiv.width * 0.5, self.__controlSwitchDiv.height * 0.5), fillcolor="00FF00", fillopacity=1.0, parent=self.__controlSwitchDiv)
        self.__controlSwitchBackground = avg.CircleNode(r=self.__controlSwitchDiv.width * 0.5, pos=(self.__controlSwitchDiv.width * 0.5, self.__controlSwitchDiv.height * 0.5), fillcolor="777777", fillopacity=1.0, parent=self.__controlSwitchDiv)
        self.__controlSwitch = avg.ImageNode(size=self.__controlSwitchDiv.size, href="../assets/HUD/balance_switch.png", parent=self.__controlSwitchDiv)
        self.__controlSwitch.subscribe(avg.Node.CURSOR_DOWN, lambda e: self.__onSwitchDown(e))
        self.__controlSwitch.subscribe(avg.Node.CURSOR_UP, lambda e: self.__onSwitchUp(e))
        self.__disabledSign = avg.LineNode(pos1=(self.__controlSwitchDiv.width - 5, 5), pos2=(5, self.__controlSwitchDiv.height - 5), strokewidth=5, color="FFFFFF", sensitive=False, parent=self.__controlSwitchDiv)
        self.__disabledSign.opacity = 0.0 if avgPlayer.balanceControlsDynamicRotation else 1.0

        self.__statusAnim = avg.LinearAnim(node=self.__switchStatus, attrName="endangle", duration=1000, startValue=0, endValue=radians(360), stopCallback=self.__onStatusFinished)
        self.__statusColorAnim = avg.LinearAnim(node=self.__switchStatus, attrName="fillcolor", duration=1000, startValue="FF0000", endValue="00FF00")

        shadowScaleFactor = imgSz.y / self.__controlShadow.size.y
        self.__controlShadow.size = avg.Point2D(self.__controlShadow.size) * shadowScaleFactor * avg.Point2D(0.76, 1)
        self.__controlShadow.pos = (self.__controlImg.pos.x - 0.5 * (self.__controlShadow.size.x - imgSz.x), self.__controlImg.pos.y)

        self.__controlLeft = avg.DivNode(pos=(0,0), size=cBtSz, parent=self.__controlsMainDiv)
        self.__controlRight = avg.DivNode(pos=(self.__controlsMainDiv.width - cBtSz.x, self.__controlLeft.pos.y), size=cBtSz, parent=self.__controlsMainDiv)

        self.__pressedControls = {}
        for i in range(self.__controlsMainDiv.getNumChildren()):
            self.__pressedControls[self.__controlsMainDiv.getChild(i)] = False

        self.__controlLeft.subscribe(avg.Node.CURSOR_DOWN, lambda e: self.__onControlDown(self.__controlLeft, e))
        self.__controlLeft.subscribe(avg.Node.CURSOR_UP, lambda e: self.__onControlUp(self.__controlLeft, e))
        self.__controlLeft.subscribe(avg.Node.CURSOR_OUT, lambda e: self.__onControlUp(self.__controlLeft, e))

        self.__controlRight.subscribe(avg.Node.CURSOR_DOWN, lambda e: self.__onControlDown(self.__controlRight, e))
        self.__controlRight.subscribe(avg.Node.CURSOR_UP, lambda e: self.__onControlUp(self.__controlRight, e))
        self.__controlRight.subscribe(avg.Node.CURSOR_OUT, lambda e: self.__onControlUp(self.__controlRight, e))

        # Cart attached indicator
        indicatorSize = (player.imageDiv.size.x * 0.75, player.imageDiv.size.y * 0.18) # assumes Player asset top is front of the cart / no node offset
        self.__indicatorMainDiv = avg.DivNode(pos=((player.imageDiv.size.x - indicatorSize[0]) * 0.5, -indicatorSize[1]), size=indicatorSize, sensitive=False, parent=player.imageDiv) # Player (Cart) attached balance indicator
        self.__indicatorRect = avg.RectNode(size=self.__indicatorMainDiv.size, color="000000", fillcolor="000000", fillopacity=0.7, opacity=0.8, parent=self.__indicatorMainDiv)
        self.__indicatorValue = avg.LineNode(pos1=(0,0), pos2=(0,indicatorSize[1]), color="FFFFFF", opacity=0.9, strokewidth=6, parent=self.__indicatorMainDiv)
        self.__indicatorHalf = self.__indicatorMainDiv.width * 0.5

        if avgPlayer.drawHelpers:
            self.__controlsMainDiv.elementoutlinecolor = "FFFF00"
            self.__indicatorMainDiv.elementoutlinecolor = "FFFF00"
            self.__mainDiv.elementoutlinecolor = "FFFF00"
            self.__controlSwitchDiv.elementoutlinecolor = "FFFF00"

    def destroy(self):
        self.__mainDiv.unlink(True)
        self.__mainDiv = None
        self.__indicatorMainDiv.unlink(True)
        self.__indicatorMainDiv = None
        while len(self.__pressedControls) > 0:
            self.__pressedControls.popitem()[0].unlink(True)

    def __onControlDown(self, controlNode, event):
        if controlNode not in self.__pressedControls:
            logger.error("Cannot Node " + str(controlNode) + "is not a valid Control Node")
            return
        self.__pressedControls[controlNode] = True
        self.__updatateControlsTexture()

    def __onControlUp(self, controlNode, event):
        if controlNode not in self.__pressedControls:
            logger.error("Node " + str(controlNode) + "is not a valid Control Node")
            return
        self.__pressedControls[controlNode] = False
        self.__updatateControlsTexture()

    def __onSwitchDown(self, e):
        self.__statusAnim.setStopCallback(self.__onStatusFinished)
        self.__statusAnim.start()
        self.__statusColorAnim.start()

    def __onStatusFinished(self):
        avgPlayer.balanceControlsDynamicRotation = not avgPlayer.balanceControlsDynamicRotation
        if not avgPlayer.balanceControlsDynamicRotation:
            self.__controlsMainDiv.angle = radians(180)
            self.__disabledSign.opacity = 1.0
        else:
            self.__disabledSign.opacity = 0.0
        self.__switchStatus.endangle = 0

    def __onSwitchUp(self, e):
        if self.__statusAnim.isRunning():
            self.__statusAnim.setStopCallback(None)
            self.__statusAnim.abort()
            self.__switchStatus.endangle = 0
        if self.__statusColorAnim.isRunning():
            self.__statusColorAnim.abort()

    def show(self, show=True):
        if not self.active and show:
            return
        if self.__mainDiv:
            self.__mainDiv.active = show
            self.__indicatorMainDiv.active = show

    def showIndicator(self, show=True):
        self.__indicatorMainDiv.active = show

    def onFrame(self, dt):
        if not self.active:
            return

        speedFactor = self.__player.speed * dt * 0.00001

        # Apply player controls
        if self.__pressedControls[self.__controlLeft]:
            self.__player.balance -= 0.001 * dt
        if self.__pressedControls[self.__controlRight]:
            self.__player.balance += 0.001 * dt

        # take curves and therefore tilted player into account
        self.__player.balance += 0.2 * speedFactor * self.__player.tilt

        # Apply acceleration
        # distinguish between linear and quadtratic increase
        balanceFactorVal = 0.05
        balanceFactor = -balanceFactorVal if self.__player.balance < 0 else balanceFactorVal # signed, allready multiplied by decent speed factor
        bAbs = abs(self.__player.balance)
        if 0.2 < bAbs < 0.8: # apply quadratic increase
            self.__player.balance += bAbs * speedFactor * balanceFactor * 2
        else: # apply linear increase
            self.__player.balance += speedFactor * balanceFactor

        # Limit values
        self.__player.balance = min(self.__player.balance, 1.0)
        self.__player.balance = max(self.__player.balance, -1.0)

        # Adjust controls angle to match ingame cart angle
        if avgPlayer.balanceControlsDynamicRotation:
            self.__controlsMainDiv.angle = self.__player.angle

        if self.__player.balance <= -0.5:
            self.__indicatorValue.color = "FF8800"
        elif self.__player.balance < 0.5:
            self.__indicatorValue.color = "FFFFFF"
        else:
            self.__indicatorValue.color = "FF8800"

        # Update indicator
        posX = self.__player.balance * self.__indicatorHalf + self.__indicatorHalf
        self.__indicatorValue.pos1 = (posX, 0)
        self.__indicatorValue.pos2 = (posX, self.__indicatorMainDiv.height)

        # Update player image offset
        self.__updatePlayerOffset()

    def __updatePlayerOffset(self):
        tileSz = Tile.getTileSize()
        maxBalanceOffset = min(tileSz[0], tileSz[1]) * 0.25
        self.__player.imageDiv.pos = avg.Point2D(self.__player.balance * maxBalanceOffset, 0)

    def resetBalance(self):
        self.__player.balance = 0.0
        self.__updatePlayerOffset()
        for node in self.__pressedControls:
            self.__pressedControls[node] = False

    # Returns balance value
    #   stable range: -1.0 to +1.0
    #   If the returned value is outside the stable range, the car is supposed to derail
    def getBalance(self):
        return self.__player.balance

    def __updatateControlsTexture(self):
        if self.__pressedControls[self.__controlLeft]:
            self.__controlImg.href = "../assets/HUD/balance_left.png"
        elif self.__pressedControls[self.__controlRight]:
            self.__controlImg.href = "../assets/HUD/balance_right.png"
        else:
            self.__controlImg.href = "../assets/HUD/balance_middle.png"
