from libavg import avg, player as avgPlayer
from item.Inventory import Inventory, Petroleum
import math

class LightControls:
    DEFAULT_LIGHT_SIZE = avg.Point2D(1400, 1400)
    PETROLEUM_LIGHT_SIZE = avg.Point2D(2500, 2500)
    PETROLEUM_LIGHT_SIZE_DIFF = PETROLEUM_LIGHT_SIZE - DEFAULT_LIGHT_SIZE
    PETROLEUM_START_UP_TIME = 2000.0 # How much time it takes to change from default light size to petroleum light size
    PETROLEUM_COOL_DOWN_TIME = 1000.0 # How much time it takes to change from petroleum light size to default light size

    def __init__(self, parentDiv, inventory):
        self.active = True
        self.__direction = avg.Point2D(0,0)

        parentDiv.subscribe(avg.Node.CURSOR_UP, self.__onLightCtrlCursorUp)
        parentDiv.subscribe(avg.Node.CURSOR_MOTION, self.__onLightCtrlCursorMotion)

        offset = 25
        lightCircleD = 250
        self.__lightCircleR = 0.5 * lightCircleD

        self.__mainDiv = avg.DivNode(pos=(parentDiv.width - lightCircleD - offset, parentDiv.height - lightCircleD - offset), size=(lightCircleD, lightCircleD), parent=parentDiv)
        self.__bgTexture = avg.ImageNode(href="../assets/HUD/lamp.png", size=self.__mainDiv.size, parent=self.__mainDiv)

        self.__lightCenterCircle = avg.CircleNode(pos=self.__mainDiv.size * 0.5, color="353535", strokewidth=2, opacity=1, fillcolor="505050", r=10, fillopacity=1, parent=self.__mainDiv)
        self.__lightCtrlCircle = avg.CircleNode(pos=self.__lightCenterCircle.pos, color="353535", strokewidth=2, opacity=1, fillcolor="505050", r=35, fillopacity=0.7, parent=self.__mainDiv)

        if avgPlayer.drawHelpers:
            self.__mainDiv.elementoutlinecolor = "FFFF00"

        self.__lightCtrlCircleOffset = self.__mainDiv.size * 0.5 + self.__mainDiv.pos

        self.__lightCtrlCircle.subscribe(avg.Node.CURSOR_DOWN, self.__onLightCtrlCursorDown)

        # Touch contact ID used for light control
        self.__lightCtrlContact = None

        self.__inventory = inventory
        self.__inventory.subscribe(Inventory.ITEM_ACTIVATED, self.__onItemActivate)
        self.__inventory.subscribe(Inventory.ITEM_DEACTIVATED, self.__onItemDeactivate)

        self.__lightSize = LightControls.DEFAULT_LIGHT_SIZE
        self.__petroleumActivated = False

    def destroy(self):
        self.__mainDiv.unlink(True)
        self.__mainDiv = None
        self.__inventory = None

    def getLightSize(self):
        return self.__lightSize

    def __onLightCtrlCursorDown(self, event):
        if self.__lightCtrlContact == None:
            self.__lightCtrlContact = event.contact

    def __onLightCtrlCursorUp(self, event):
        if self.__lightCtrlContact == event.contact:
            self.__lightCtrlContact = None

    def __onLightCtrlCursorMotion(self, event):
        if (self.__lightCtrlContact != None) and (event.contact == self.__lightCtrlContact):

            a = event.pos - self.__lightCtrlCircleOffset
            radius = a.getNorm()
            if radius > self.__lightCircleR:
                radius = self.__lightCircleR

            a = (avg.Point2D(0,0) if radius < 0.0001 else a.getNormalized() * radius)

            self.__lightCtrlCircle.pos = self.__lightCenterCircle.pos + a
            self.__direction = a

    def show(self, show=True):
        if not self.active and show:
            return
        if self.__mainDiv:
            self.__mainDiv.active = show

    def onFrame(self, dt):
        # Calculate light direction
        if self.__lightCtrlContact == None:
            radius = self.__direction.getNorm()
            radius = max(0, radius - 0.2 * dt)
            self.__direction = (avg.Point2D(0,0) if radius < 0.0001 else self.__direction.getNormalized() * radius)
            self.__lightCtrlCircle.pos = self.__lightCenterCircle.pos + self.__direction

        # Calculate light size
        if self.__petroleumActivated:
            if self.__lightSize.x < LightControls.PETROLEUM_LIGHT_SIZE.x: # Limit max size
                step = (dt / LightControls.PETROLEUM_START_UP_TIME) * LightControls.PETROLEUM_LIGHT_SIZE_DIFF
                self.__lightSize += step

        else:
            if self.__lightSize.x > LightControls.DEFAULT_LIGHT_SIZE.x: # Limit min size
                step = (dt / LightControls.PETROLEUM_COOL_DOWN_TIME) * LightControls.PETROLEUM_LIGHT_SIZE_DIFF
                self.__lightSize -= step

    # Returns the direction vector from the circle center to the
    # light position as a Point2D
    def getDirection(self):
        return self.__direction

    def __onItemActivate(self, item):
        if item.__class__ == Petroleum:
            self.__petroleumActivated = True

    def __onItemDeactivate(self, item):
        if item.__class__ == Petroleum:
            self.__petroleumActivated = False
