from libavg import avg, logger, player as avgPlayer
from World import World

class Minimap():
    minimapHeight = 220

    def __init__(self, world, parentDiv):
        self.active = True
        # Minimap
        self.__objectivePos = world.getObjectivePos()
        self.__minimapSz = avg.Point2D(world.getSize()[0] * (Minimap.minimapHeight / world.getSize()[1]), Minimap.minimapHeight)
        texSz = avg.Point2D(self.__minimapSz.x * 1.336, self.__minimapSz.y * 1.309) # floats are ratio image dimensions to inner image space
        texOffset = avg.Point2D(-texSz.x * 0.133, -texSz.y * 0.134) # procentual offset of actual inner image space

        self.__tileSz = avg.Point2D(self.__minimapSz.x / world.getSize()[0], self.__minimapSz.y / world.getSize()[1])
        self.__tileSzHalf = avg.Point2D(self.__tileSz.x * 0.5, self.__tileSz.y * 0.5)
        self.__tilePosCache = []

        self.__mainDiv = avg.DivNode(pos=(25 - texOffset.x, parentDiv.height - 25 - self.__minimapSz.y + texOffset.y), size=self.__minimapSz, sensitive=False, parent=parentDiv)
        self.__bgTexture = avg.ImageNode(href="../assets/HUD/map_bg.png", size=texSz, pos=texOffset ,parent=self.__mainDiv)
        self.__minimaptilesDiv = avg.DivNode(parent=self.__mainDiv)
        self.__minimapPlayer = avg.CircleNode(pos=(-1000, -1000), r=min(self.__tileSz.x, self.__tileSz.y), color="353535", fillcolor="505050", fillopacity=1, parent=self.__mainDiv) # not visible at startup

        # Target Indicator
        self.__maxminimapVectorLength = self.__minimapSz.getNorm() * 0.5 + 0.1 # center of the map to any corner
        self.__minimapCenter = self.__minimapSz * 0.5
        self.__worldPixelSize = World.getPixelPos(world.getSize())
        self.__maxDistance = self.__worldPixelSize.getNorm()
        self.__targetPos = World.getPixelPos(world.getObjectivePos())
        self.__directionIndicatorDefaultLength = min(self.__minimapSz.x, self.__minimapSz.y)

        self.__intersectionPoint = self.__minimapCenter # intersection between minimap border and vector from center to finalPos
        self.__directionMarker = avg.PolyLineNode(color="9900FF", strokewidth=3, opacity=1, active=False, parent=self.__mainDiv) # highlights complete border (top, right, bot or left)
        self.__directionIndicator = avg.PolyLineNode(color="39C350", strokewidth=3, opacity=1, parent=self.__mainDiv) # highlights targetdirection, length shorter zthe closer to objective

        self.__directionHelper = avg.LineNode(pos1=self.__minimapCenter, pos2=(-1000, -1000), strokewidth=2, color="0000FF", active=False, parent=self.__mainDiv) # straight, pointing from minimap center to intersection point
        self.__distanceString = avg.WordsNode(pos=(self.__minimapSz.x + 10,self.__minimapSz.y - 60), text="n/a", fontsize=60, active=False, parent=self.__mainDiv)

        pos = avg.Point2D(self.__objectivePos[0] * self.__tileSz.x, self.__objectivePos[1] * self.__tileSz.y)
        self.__objectiveNode = avg.RectNode(pos=pos, size=self.__tileSz, color="ff6600", fillcolor="ff6600", fillopacity=0.0, opacity=0.0, parent=self.__minimaptilesDiv)
        self.__objectiveCircle = avg.CircleNode(r=5, pos=self.__objectiveNode.pos + (self.__tileSzHalf.x, self.__tileSzHalf.y), opacity=0.0, color="ff6600", strokewidth=3, parent=self.__minimaptilesDiv)
        self.__fadeOutOjectiveNode = avg.LinearAnim(node=self.__objectiveNode, attrName="fillopacity", startValue=1.0, endValue=0.0, duration=500, stopCallback=self.__handleBlinking)
        self.__circleRadiusAnim = avg.LinearAnim(node=self.__objectiveCircle, attrName="r", duration=900, startValue=5, endValue=300)
        self.__fadeOutObjectiveCircle = avg.LinearAnim(node=self.__objectiveCircle, attrName="opacity", startValue=1.0, endValue=0.0, duration=900)
        self.__fadeInObjectiveNode = avg.LinearAnim(self.__objectiveNode, attrName="fillopacity", startValue=0.0, endValue=1.0, duration=500, stopCallback=self.__startBlinking)
        self.__animCount = 0

        if avgPlayer.drawHelpers:
            self.__mainDiv.elementoutlinecolor = "FFFF00"
            self.__distanceString.active = True
            self.__directionHelper.active = True
            self.__directionMarker.active = True

    def destroy(self):
        self.__mainDiv.unlink(True)
        self.__mainDiv = None

    def show(self, show=True):
        if not self.active and show:
            return
        if self.__mainDiv:
            self.__mainDiv.active = show

    def ping(self):
        self.__objectiveNode.opacity = 1.0
        self.__objectiveNode.fillopacity = 1.0
        self.__objectiveCircle.opacity = 1.0
        self.__fadeOutOjectiveNode.stopCallback = self.__handleBlinking
        self.__fadeInObjectiveNode.stopCallback = self.__startBlinking
        if not (self.__fadeOutOjectiveNode.isRunning() or self.__circleRadiusAnim.isRunning() or self.__fadeOutObjectiveCircle.isRunning()):
            self.__startBlinking()

    def __startBlinking(self):
        self.__fadeOutOjectiveNode.start()
        self.__circleRadiusAnim.start()
        self.__fadeOutObjectiveCircle.start()

    def __handleBlinking(self):
        if self.__animCount < 4:
            self.__fadeInObjectiveNode.start()
            self.__animCount += 1
        else:
            self.__animCount = 0
            self.__fadeOutOjectiveNode.stopCallback = None
            self.__fadeInObjectiveNode.stopCallback = None
            self.__fadeOutOjectiveNode.abort()
            self.__fadeInObjectiveNode.abort()
            self.__circleRadiusAnim.abort()
            self.__fadeOutObjectiveCircle.abort()
            self.__objectiveNode.opacity = 0.0
            self.__objectiveNode.fillopacity = 0.0
            self.__objectiveCircle.opacity = 0.0


    def getSize(self):
        return self.__mainDiv.size

    def addTile(self, tilePos):
        if tilePos not in self.__tilePosCache:
            pos = avg.Point2D(tilePos[0] * self.__tileSz.x, tilePos[1] * self.__tileSz.y)
            avg.RectNode(pos=pos, size=self.__tileSz, color="8C8C8C",fillcolor="8C8C8C", fillopacity=1, parent=self.__minimaptilesDiv)
            self.__tilePosCache.append(tilePos)

    def onFrame(self, playerPos):
        # Minimap
        playerPosRatio = avg.Point2D(playerPos.x / self.__worldPixelSize.x, playerPos.y / self.__worldPixelSize.y)
        self.__minimapPlayer.pos = (playerPosRatio.x * self.__minimapSz.x + self.__tileSzHalf.x, playerPosRatio.y * self.__minimapSz.y + self.__tileSzHalf.y)

        # Target Indicator
        vec = self.__targetPos - playerPos # current player position to target position
        distance = (self.__targetPos - playerPos).getNorm() # calculate absolute distance to target Tile
        vecNormalized = (vec / distance if distance > 0.0001 else avg.Point2D(0,0))
        vecTest = self.__minimapCenter + vecNormalized * self.__maxminimapVectorLength # vector used for border intersection tests

        # calculate intensity
        currentDistanceRatio = distance / self.__maxDistance
        distanceIndicatorHalf = self.__directionIndicatorDefaultLength * currentDistanceRatio * 0.5

        # project direction on minimap border to get reference point of direction Indicator
        # check for corners and bend indicator if needed
        # very ugly solution, any improvement welcome

        # top
        if vecTest.y < 0.0:
            self.__directionMarker.pos = ((0,0),(self.__minimapSz.x,0))
            self.__intersectionPoint = self.__calcIntersection((self.__minimapCenter, vecNormalized),(avg.Point2D(0, 0),avg.Point2D(1, 0)))

            tmp = self.__intersectionPoint.x + distanceIndicatorHalf
            if tmp > self.__minimapSz.x:
                self.__directionIndicator.pos = [self.__intersectionPoint - avg.Point2D(distanceIndicatorHalf, 0), (self.__minimapSz.x,0), avg.Point2D(self.__minimapSz.x, tmp - self.__minimapSz.x)]

            else:
                tmp = self.__intersectionPoint.x - distanceIndicatorHalf
                if tmp < 0:
                    self.__directionIndicator.pos = [self.__intersectionPoint + avg.Point2D(distanceIndicatorHalf, 0), (0,0), (0, -tmp)]
                else:
                    self.__directionIndicator.pos = [self.__intersectionPoint + avg.Point2D(distanceIndicatorHalf, 0),self.__intersectionPoint - avg.Point2D(distanceIndicatorHalf, 0)]

        # bottom
        elif vecTest.y > self.__minimapSz.y:
            self.__directionMarker.pos = ((0,self.__minimapSz.y),(self.__minimapSz.x,self.__minimapSz.y))
            self.__intersectionPoint = self.__calcIntersection((self.__minimapCenter, vecNormalized),(avg.Point2D(0, self.__minimapSz.y),avg.Point2D(1, 0)))

            tmp = self.__intersectionPoint.x + distanceIndicatorHalf
            if tmp > self.__minimapSz.x:
                self.__directionIndicator.pos = [self.__intersectionPoint - avg.Point2D(distanceIndicatorHalf, 0), self.__minimapSz, self.__minimapSz - avg.Point2D(0, tmp - self.__minimapSz.x)]

            else:
                tmp = self.__intersectionPoint.x - distanceIndicatorHalf
                if tmp < 0:
                    self.__directionIndicator.pos = [self.__intersectionPoint + avg.Point2D(distanceIndicatorHalf, 0), (0,self.__minimapSz.y), (0, self.__minimapSz.y + tmp)]
                else:
                    self.__directionIndicator.pos = [self.__intersectionPoint + avg.Point2D(distanceIndicatorHalf, 0),self.__intersectionPoint - avg.Point2D(distanceIndicatorHalf, 0)]

        # left
        elif vecTest.x < 0.0:
            self.__directionMarker.pos = ((0,0),(0,self.__minimapSz.y))
            self.__intersectionPoint = self.__calcIntersection((self.__minimapCenter, vecNormalized),(avg.Point2D(0, 0),avg.Point2D(0, 1)))

            tmp = self.__intersectionPoint.y + distanceIndicatorHalf
            if tmp > self.__minimapSz.y:
                self.__directionIndicator.pos = [self.__intersectionPoint - avg.Point2D(0, distanceIndicatorHalf), avg.Point2D(0, self.__minimapSz.y), avg.Point2D(tmp - self.__minimapSz.y, self.__minimapSz.y)]

            else:
                tmp = self.__intersectionPoint.y - distanceIndicatorHalf
                if tmp < 0:
                    self.__directionIndicator.pos = [self.__intersectionPoint + avg.Point2D(0, distanceIndicatorHalf), (0, 0), (-tmp, 0)]
                else:
                    self.__directionIndicator.pos = [self.__intersectionPoint + avg.Point2D(0, distanceIndicatorHalf),self.__intersectionPoint - avg.Point2D(0, distanceIndicatorHalf)]

        # right
        elif vecTest.x > self.__minimapSz.x:
            self.__directionMarker.pos = ((self.__minimapSz.x,0),(self.__minimapSz.x,self.__minimapSz.y))
            self.__intersectionPoint = self.__calcIntersection((self.__minimapCenter, vecNormalized),(avg.Point2D(self.__minimapSz.x, 0),avg.Point2D(0,1)))

            tmp = self.__intersectionPoint.y + distanceIndicatorHalf
            if tmp > self.__minimapSz.y:
                self.__directionIndicator.pos = [self.__intersectionPoint - avg.Point2D(0, distanceIndicatorHalf), self.__minimapSz, self.__minimapSz - avg.Point2D(tmp - self.__minimapSz.y, 0)]

            else:
                tmp = self.__intersectionPoint.y - distanceIndicatorHalf
                if tmp < 0:
                    self.__directionIndicator.pos = [self.__intersectionPoint + avg.Point2D(0, distanceIndicatorHalf), (self.__minimapSz.x, 0), (self.__minimapSz.x + tmp, 0)]
                else:
                    self.__directionIndicator.pos = [self.__intersectionPoint + avg.Point2D(0, distanceIndicatorHalf),self.__intersectionPoint - avg.Point2D(0, distanceIndicatorHalf)]

        else:
            logger.error("Could not calculate minimap Intersection Point")
            return

        self.__directionHelper.pos2 = self.__intersectionPoint
        self.__distanceString.text = str(round(distance, 1)) + " | " + str(round(currentDistanceRatio, 2))

    # Takes 2 vectors: vecX = (positionVectorX,directVectorX)
    # Returns position Vector if intersection is existend, else returns None
    def __calcIntersection(self, vec1, vec2):
        directionVectorCross = self.__crossProduct2(vec1[1], vec2[1])
        if directionVectorCross == 0:
            return None
        factorDirectionVector1 = self.__crossProduct2(vec2[0] - vec1[0], vec2[1]) / directionVectorCross
        return vec1[0] + factorDirectionVector1 * vec1[1]


    def __crossProduct2(self, vec1, vec2):
        return vec1[0] * vec2[1] - vec1[1] * vec2[0]
