# coding=utf-8
from libavg import avg, player as avgPlayer
from HighScore import HighScore
from menu.ScreenMessenger import ScreenMessenger
from menu.StyleDefinitions import StyleDefinitions
from menu.Button import Button

class CampaignFinishScreen(avg.DivNode):

    SCREEN_CLOSED = avg.Publisher.genMessageID()

    def __init__(self, parentNode):
        super(CampaignFinishScreen, self).__init__()
        self.registerInstance(self, parentNode)

        self.publish(CampaignFinishScreen.SCREEN_CLOSED)

        self.__campaignFinishScreen = avg.DivNode(size=parentNode.size, parent=self)

        highScoreRelHeight = 0.7
        self.__bgColor = avg.RectNode(size=self.__campaignFinishScreen.size, color="000000", fillcolor="000000", fillopacity=1, parent=self.__campaignFinishScreen)
        self.__bgImage = avg.ImageNode(size=self.__campaignFinishScreen.size, href=StyleDefinitions.MENU_BG_IMAGE, parent=self.__campaignFinishScreen)

        # HighScore Listing without current score
        self.__highscore = ScreenMessenger(size=(self.__campaignFinishScreen.width, self.__campaignFinishScreen.height * highScoreRelHeight), headline="Highscore", message=HighScore.getHumanReadableHighscoreEntries(), destroyOnFinish=True, messageAlignment="center", headlineAlignment="center", drawBG=False, button=False, parent=self.__campaignFinishScreen)

        # Current Score and placement, team name input
        score = avgPlayer.statisticsManager.getCompleteTime()

        textColor = self.__highscore.getFontColor()
        fontSize = self.__highscore.getFontSize()

        self.__highscoreForm = avg.DivNode(pos=(0, self.__highscore.height), size=(self.__campaignFinishScreen.width, self.__campaignFinishScreen.height - self.__highscore.height), parent=self.__campaignFinishScreen)

        ranking = HighScore.getPossibleHighscorePlacement(score)
        scoreString = "Euer Score von <b>" + str(round(score * 0.001, 1)) + "</b> Sekunden"
        if ranking is not None:
            scoreString += " sichert euch Platz " + str(ranking + 1) + ".<br/> Herzlichen Glückwunsch!"
        else:
            scoreString += " ist leider zu gering für den Highscore."

        scoreLabel = avg.WordsNode(pos=(self.__highscoreForm.width * 0.5, 0), font=StyleDefinitions.FONT_PARAGRAPH, text=scoreString, fontsize=fontSize, color=textColor, alignment="center", parent=self.__highscoreForm)
        self.__keyDownSubscriberID = None

        inputSz = avg.Point2D(fontSize * 14.5, fontSize * 1.2)
        inputFormSz = inputSz
        buttonSz = avg.Point2D(300, inputSz.y)

        if ranking is not None:

            # Text Input field
            inputForm = avg.DivNode(pos=(self.__highscoreForm.width * 0.5 - inputFormSz[0] * 0.5, scoreLabel.pos.y + scoreLabel.height + fontSize * 0.5), size=inputFormSz, parent=self.__highscoreForm)

            textInputDiv = avg.DivNode(size=inputSz, parent=inputForm)
            textInputBg = avg.RectNode(size=textInputDiv.size, color="CCCCCC", fillcolor="CCCCCC", fillopacity=0.3, parent=textInputDiv)
            self.__textInput = avg.WordsNode(pos=(10, -fontSize * 0.1), font=StyleDefinitions.FONT_PARAGRAPH, fontsize=fontSize, color=textColor, parent=textInputDiv)
            self.__textInputPlaceholder = avg.WordsNode(pos=self.__textInput.pos, font=self.__textInput.font, text="Hier Team Namen eintragen", fontsize=fontSize, opacity=0.7, color=textColor, parent=textInputDiv)

            self.__keyDownSubscriberID = avgPlayer.subscribe(avgPlayer.KEY_DOWN, lambda e: self.__inputTeamName(e))

            cursorWidth = 4
            cursorX = self.__textInput.pos.x + cursorWidth * 0.5 + 1
            cursorHeightHalf = fontSize * 0.5
            inputHeightHalf = textInputDiv.height * 0.5
            self.__cursor = avg.LineNode(pos1=(cursorX, inputHeightHalf - cursorHeightHalf), pos2=(cursorX, inputHeightHalf + cursorHeightHalf), color=textColor, strokewidth=cursorWidth, parent=textInputDiv)
            self.__cursorAnim = avg.EaseInOutAnim(node=self.__cursor, attrName="opacity", duration=1000, startValue=1, endValue=0.1, easeInDuration=100, easeOutDuration=100, stopCallback=lambda: self.__cursorAnim.start())
            self.__cursorAnim.start()

        buttonsSz = avg.Point2D(2.5 * buttonSz.x, buttonSz.y)
        self.__buttons = avg.DivNode(pos=(self.__campaignFinishScreen.width * 0.5 - buttonsSz.x * 0.5, self.__campaignFinishScreen.height - buttonSz.y - fontSize * 0.5), size=buttonsSz, parent=self.__campaignFinishScreen)

        # Continue Button
        continueButton = Button(pos=(self.__buttons.width - buttonSz.x, 0), size=buttonSz, text="WEITER", textColor=StyleDefinitions.FONT_COLOR, parent=self.__buttons)
        if ranking is not None:
            continueButton.subscribe(Button.BUTTON_CLICKED, lambda: self.__close(self.__textInput.text))
        else:
            continueButton.subscribe(Button.BUTTON_CLICKED, self.__close)

        # Statistics Button
        statisticsButton = Button(size=buttonSz, text="STATISTIK", textColor=StyleDefinitions.FONT_COLOR, parent=self.__buttons)
        statisticsButton.subscribe(Button.BUTTON_CLICKED, self.__showStatistics)

    def __close(self, teamName=None):
        if self.__keyDownSubscriberID:
            avgPlayer.unsubscribe(avgPlayer.KEY_DOWN,self.__keyDownSubscriberID)

        self.__campaignFinishScreen.unlink(True)
        self.__campaignFinishScreen = None

        if isinstance(teamName, unicode) or isinstance(teamName, str):
            if teamName == "":
                teamName = "kein_Team_Name"
            HighScore.publishScore(teamName, avgPlayer.statisticsManager.getCompleteTime())
            self.notifySubscribers(CampaignFinishScreen.SCREEN_CLOSED, [])

        super(CampaignFinishScreen, self).unlink(True)

    def __inputTeamName(self, event):
        key = event.text
        if (key.isalnum() or key in ["-", "_", "."]) and len(self.__textInput.text) <= 16:
            self.__textInputPlaceholder.active = False
            self.__textInput.text += key
        elif event.keyname == "Backspace":
            self.__textInput.text = self.__textInput.text[:-1]

        cursorX = self.__textInput.pos.x + self.__cursor.strokewidth * 0.5 + 1
        if self.__textInput.text:
            cursorX += self.__textInput.width
        self.__cursor.pos1=(cursorX, self.__cursor.pos1.y)
        self.__cursor.pos2=(cursorX, self.__cursor.pos2.y)

    def __showStatistics(self):
        self.__statistics = ScreenMessenger(size=self.__campaignFinishScreen.size, headline="Statistiken", message=str(avgPlayer.statisticsManager.getHumanReadableStatistics()), destroyOnFinish=True, buttonText="ZURÜCK", parent=self.__campaignFinishScreen)
