# coding=utf-8
from sys import maxint
from libavg import avg, player as avgPlayer
from HighScore import HighScore
from menu.ScreenMessenger import ScreenMessenger
from menu.CampaignFinishScreen import CampaignFinishScreen
from menu.Button import Button
from menu.StyleDefinitions import StyleDefinitions
import random

class Menu():

    def __init__(self, parentNode):

        ASSET_DIR = "../assets/menu/"
        self.reset()

        self.__menuLayer = avg.DivNode(pos=(0,0), size=parentNode.size, parent=parentNode)
        menuSz = self.__menuLayer.size

        self.__menuBackground = avg.ImageNode(size=menuSz, href=ASSET_DIR + "background.jpg", parent=self.__menuLayer)

        # MENU LAYOUT:
        # first vertical quarter: Game Title (integrated in background image)
        # Buttons in own container below

        # Title Banner
        self.__bannerDiv = avg.DivNode(size=(menuSz.x, menuSz.y * 0.25), parent=self.__menuLayer)

        # Buttons
        buttonOpacity = 0.7
        self.__buttonDiv = avg.DivNode(pos=(0, self.__bannerDiv.height), size=(self.__menuLayer.width, self.__menuLayer.height * 0.75), parent=self.__menuLayer)

        self.__campaignButton = Button(text="Kampagne", textColor=StyleDefinitions.FONT_COLOR, backgroundColor=StyleDefinitions.BACKGROUND_COLOR, opacity=buttonOpacity, bgImageHref=ASSET_DIR + "wood_1.png", parent=self.__buttonDiv)
        self.__campaignButton.subscribe(Button.BUTTON_CLICKED, self.__setCampaign)

        self.__quickplayButton = Button(text="Schnelles Spiel", textColor=StyleDefinitions.FONT_COLOR, backgroundColor=StyleDefinitions.BACKGROUND_COLOR, opacity=buttonOpacity, bgImageHref=ASSET_DIR + "wood_2.png", parent=self.__buttonDiv)
        self.__quickplayButton.subscribe(Button.BUTTON_CLICKED, self.__setQuickplay)

        self.__tutorialButton = Button(text="Tutorial", textColor=StyleDefinitions.FONT_COLOR, backgroundColor=StyleDefinitions.BACKGROUND_COLOR, opacity=buttonOpacity, bgImageHref=ASSET_DIR + "wood_3.png", parent=self.__buttonDiv)
        self.__tutorialButton.subscribe(Button.BUTTON_CLICKED, self.__setTutorial)

        self.__highscoreButton = Button(text="Highscore", textColor=StyleDefinitions.FONT_COLOR, backgroundColor=StyleDefinitions.BACKGROUND_COLOR, opacity=buttonOpacity, bgImageHref=ASSET_DIR + "wood_4.png", parent=self.__buttonDiv)
        self.__highscoreButton.subscribe(Button.BUTTON_CLICKED, self.displayHighscore)

        self.__quitButton = Button(text="Beenden", textColor=StyleDefinitions.FONT_COLOR, backgroundColor=StyleDefinitions.BACKGROUND_COLOR, opacity=buttonOpacity, bgImageHref=ASSET_DIR + "wood_5.png", parent=self.__buttonDiv)
        self.__quitButton.subscribe(Button.BUTTON_CLICKED, quit)

        # Set Button sizes
        relMarginYOut = 1.6 # margin at top and bottom (relative to button height)
        relMarginY = 0.4 # margin between buttons (relative to button height)
        buttonCount = self.__buttonDiv.getNumChildren()
        buttonHeight = self.__buttonDiv.height / ((buttonCount -1) * (1 + relMarginY) + 1 + relMarginYOut * 2)
        buttonWidth = (StyleDefinitions.BUTTON_LARGE_SIZE.x / StyleDefinitions.BUTTON_LARGE_SIZE.y) * buttonHeight

        # Spread buttons evenly
        minFontSize = maxint
        for i in xrange(buttonCount):
            button = self.__buttonDiv.getChild(i)
            button.setSize(avg.Point2D(buttonWidth, buttonHeight))
            button.pos = (25, buttonHeight * (1 + relMarginY) * i + (relMarginYOut) * buttonHeight)
            minFontSize = min(minFontSize, button.getTextSize())

        # set largest possible fontsize for all buttons
        for i in xrange(buttonCount):
            button = self.__buttonDiv.getChild(i)
            button.setTextSize(minFontSize)


    def reset(self):
        self.startCampaign = False
        self.startQuickplay = False
        self.startTutorial = False

    def showMenu(self, show=True):
        self.__menuLayer.active = show

    def hideMenu(self):
        self.showMenu(False)

    def __setCampaign(self):
        self.startCampaign = True

    def __setQuickplay(self):
        self.startQuickplay = True

    def __setTutorial(self):
        self.startTutorial = True

    def onFrame(self):
        pass

    def displayHighscore(self):
        highscore = ScreenMessenger(size=self.__menuLayer.size, headline="Highscore", message=HighScore.getHumanReadableHighscoreEntries(), destroyOnFinish=True, buttonText="ZURÜCK", messageAlignment="center", parent=self.__menuLayer)

    def displayCampaignFinishScreen(self):
        self.__campaignFinishScreen = CampaignFinishScreen(self.__menuLayer)
        self.__campaignFinishScreen.subscribe(CampaignFinishScreen.SCREEN_CLOSED, self.displayHighscore)
