from libavg import avg, logger, player as avgPlayer
from Util import *

class MenuTransition:
    DEFAULT_FADE_DURATION = 1500 # ms

    def __init__(self, parentNode):
        self.__overlay = avg.RectNode(
            size=parentNode.size,
            fillcolor="000000",
            fillopacity=0.0,
            strokewidth=0,
            sensitive=False,
            parent=parentNode)

        self.__anim = None
        self.__stopCallback = None
        self.__timeout = None

    # Returns False if there is already a fade animation running, otherwise True
    # duration - milliseconds
    # startDelay - milliseconds
    def startFadeAnim(self, fadeIn=True, stopCallback=None, duration=None, startDelay=0):
        if duration is None:
            duration = MenuTransition.DEFAULT_FADE_DURATION

        if self.__anim is not None:
            logger.error("Cannot start transition fade animation: A transition is already running!")
            return False

        bringNodeToTop(self.__overlay)
        self.__stopCallback = stopCallback

        # Do not allow input during transition
        self.__overlay.sensitive = True

        finalOpacity = (0.0 if fadeIn else 1.0)
        startOpacity = 1.0 - finalOpacity
        self.__overlay.fillopacity = startOpacity
        self.__anim = avg.LinearAnim(self.__overlay, "fillopacity", duration, startOpacity, finalOpacity, stopCallback=self.__onAnimStopped)
        self.__timeout = avgPlayer.setTimeout(startDelay, self.__onStartTimeoutFinished)
        return True

    def startFadeIn(self, stopCallback=None, duration=None, startDelay=0):
        return self.startFadeAnim(True, stopCallback, duration, startDelay)

    def startFadeOut(self, stopCallback=None, duration=None, startDelay=0):
        return self.startFadeAnim(False, stopCallback, duration, startDelay)

    def abort(self):
        if self.__timeout is not None:
            # yes, clearInterval() - clearTimeout() does not exist.
            avgPlayer.clearInterval(self.__timeout)
            self.__timeout = None

        if self.isRunning():
            self.__anim.abort()
            self.__anim = None

        self.__stopCallback = None
        self.__overlay.sensitive = False

    def isRunning(self):
        # Note that self.__anim.isRunning() maybe false while this transition
        # is already running during start delay
        return self.__anim is not None

    def __onStartTimeoutFinished(self):
        self.__timeout = None
        self.__anim.start()

    def __onAnimStopped(self):
        self.__anim.abort()
        self.__anim = None
        self.__overlay.sensitive = False
        if self.__stopCallback is not None:
            self.__stopCallback()
            self.__stopCallback = None
