from libavg import avg, logger
from menu.StyleDefinitions import StyleDefinitions
from menu.Button import Button


# Class for printing messages on the Screen
class ScreenMessenger(avg.DivNode):

    def __init__(self, pos=(0,0), size=(800, 600), headline="Headline", message="Text", fontcolor=StyleDefinitions.FONT_COLOR, backgroundcolor=StyleDefinitions.BACKGROUND_COLOR, parent=None, destroyOnFinish=False, margin=True, messageAlignment="left", headlineAlignment="center", button=True, buttonText="WEITER", additionalMessageNodes=[], drawBG=True, bgImageHref=StyleDefinitions.MENU_BG_IMAGE):
        super(ScreenMessenger, self).__init__(pos=pos, size=size)
        self.registerInstance(self, parent)
        self.headline = headline
        self.message = message

        self.__finished = False # Whether this message has been finidhed (button pressed ... etc)
        self.__destroyOnFinish = destroyOnFinish # Weather this Instance should destroy itself once it is finished

        if margin == True:
            marginRelX = 0.06
            marginRelY = 0.02
        else:
            marginRelX = 0.0
            marginRelY = 0.0

        drawLayout = False

        self.__mainDiv = avg.DivNode(size=self.size, parent=self)

        self.__bgColor = avg.RectNode(size=self.__mainDiv.size, color="000000", fillcolor="000000", fillopacity=1.0, parent=self.__mainDiv)
        self.__bgImage = avg.RectNode(size=self.__mainDiv.size, fillcolor=backgroundcolor, filltexhref=bgImageHref, color=backgroundcolor, opacity=0.0, fillopacity=1.0, parent=self.__mainDiv)
        if not drawBG:
            self.__bgColor.active = False
            self.__bgImage.active = False

        self.__contentDiv = avg.DivNode(size=((1 - marginRelX * 2) * self.__mainDiv.size.x, (1 - marginRelY * 2) * self.__mainDiv.size.y), pos=(self.__mainDiv.size.x * marginRelX, self.__mainDiv.size.y * marginRelY), crop=True, parent=self.__mainDiv)

        cWH = self.__contentDiv.size.x * 0.5 # ContentWidthHalf
        headRelHeight = 0.125

        self.__headDiv = avg.DivNode(size=(self.__contentDiv.size.x, self.__contentDiv.size.y * headRelHeight), parent=self.__contentDiv)
        headlinePos = (self.__headDiv.size.x * 0.5,0)
        if headlineAlignment == "left":
            headlinePos = (0,0)
        elif headlineAlignment == "right":
            headlinePos = (self.__headDiv.width,0)
        self.__headline = avg.WordsNode(font=StyleDefinitions.FONT_HEADLINE, text=self.headline, pos=headlinePos, color=fontcolor, fontsize=self.__headDiv.size.x / 14, alignment=headlineAlignment, parent=self.__headDiv)

        padding = self.__headline.fontsize * 0.5 # absolute vertical distance between headline, message and button

        if button:
            btnSz = StyleDefinitions.BUTTON_LARGE_SIZE
            self.__continueButton = Button(pos=(cWH - btnSz.x * 0.5, self.__contentDiv.size.y - btnSz.y - padding * 0.5), text=buttonText, size=btnSz, textColor=StyleDefinitions.FONT_COLOR, parent=self.__contentDiv)
            self.__continueButton.subscribe(Button.BUTTON_CLICKED, self.__onButtonPress)
        else:
            btnSz = avg.Point2D(0, 0)
            self.__continueButton = avg.DivNode(pos=(cWH - btnSz.x * 0.5, self.__contentDiv.size.y - btnSz.y), size=btnSz, active=False, parent=self.__contentDiv)

        self.__messageDiv = avg.DivNode(pos=(0, self.__headDiv.size.y + padding), size=(self.__contentDiv.size.x, self.__contentDiv.size.y * (1 - headRelHeight) - self.__continueButton.size.y - padding * 2), crop=True, parent=self.__contentDiv)
        messagePos = (0, 0)
        messageWidth = self.__messageDiv.size.x
        if messageAlignment == "center":
            messagePos = (self.__messageDiv.size.x * 0.5, 0)
            messageWidth = 0
        elif messageAlignment == "right":
            messagePos = (self.__messageDiv.size.x, 0)
            messageWidth = 0
        self.__message = avg.DivNode(width=self.__messageDiv.width, parent=self.__messageDiv)
        self.__messageText = avg.WordsNode(font=StyleDefinitions.FONT_PARAGRAPH, pos=messagePos, width=messageWidth, text=self.message, color=fontcolor, fontsize=min(self.__messageDiv.size.x / 18, self.__messageDiv.size.y / 12), alignment=messageAlignment, sensitive=False, parent=self.__message)

        # Append Nodes below the message in separate container
        self.__additionalMessageNodesDiv = avg.DivNode(pos=(0, self.__messageText.size.y), width=self.__messageDiv.width, parent=self.__message)
        aMNM = 10 # additionalMessageNodesMargin (y)
        lastAdditionalNode = None
        for node in additionalMessageNodes:
            if not isinstance(node, avg.AreaNode):
                continue
            # Resize if needed
            ratio = self.__additionalMessageNodesDiv.width / node.width
            if ratio < 1.0:
                node.size = (self.__additionalMessageNodesDiv.width, node.size.y * ratio)

            if lastAdditionalNode:
                node.pos = (self.__additionalMessageNodesDiv.width * 0.5 - node.size.x * 0.5, lastAdditionalNode.pos.y + lastAdditionalNode.size.y + aMNM)
            else:
                node.pos = (self.__additionalMessageNodesDiv.width * 0.5 - node.size.x * 0.5, aMNM)

            self.__additionalMessageNodesDiv.appendChild(node)
            self.__additionalMessageNodesDiv.height += aMNM + node.height
            lastAdditionalNode = node

        self.__message.height = self.__messageText.height + self.__additionalMessageNodesDiv.height

        self.__message.down = False
        self.__messageDiv.subscribe(avg.Node.CURSOR_UP, lambda e: self.__onMessageUp(e))
        self.__messageDiv.subscribe(avg.Node.CURSOR_DOWN, lambda e: self.__onMessageDown(e))
        self.__messageDiv.subscribe(avg.Node.CURSOR_MOTION, lambda e: self.__onMessageMotion(e))
        self.__messageDiv.subscribe(avg.Node.CURSOR_OUT, lambda e: self.__onMessageUp(e))

        if drawLayout:
            self.__mainBG = avg.RectNode(size=self.size, sensitive=False, parent=self.__mainDiv)
            self.__contentBg = avg.RectNode(size=self.__contentDiv.size, sensitive=False, parent=self.__contentDiv)
            self.__headBg = avg.RectNode(size=self.__headDiv.size, parent=self.__headDiv)
            self.__messageBg = avg.RectNode(size=self.__messageDiv.size, parent=self.__messageDiv)
            self.__messageDiv.elementoutlinecolor="FF0000"

    def __onButtonPress(self):
        self.__finished = True
        if self.__destroyOnFinish:
            self.unlink()

    def __onMessageDown(self, event):
        self.__message.down = True
        self.__message.downPos = event.pos
        self.__message.lastPos = self.__message.pos

    def __onMessageMotion(self, event):
        if self.__message.down:
            diffY = event.pos.y - self.__message.downPos.y
            newYPos = self.__message.lastPos.y + diffY

            # Scrolling Upwards
            if diffY < 0:
                if self.__message.size.y + self.__message.pos.y > self.__messageDiv.height:
                    newYPos = max(self.__messageDiv.height - self.__message.size.y, newYPos)
                    self.__message.pos = (self.__message.pos.x, newYPos)
            # Scrolling downwards
            elif self.__message.pos.y < 0:
                    newYPos = min(0, newYPos)
                    self.__message.pos = (self.__message.pos.x, newYPos)

    def __onMessageUp(self, event):
        self.__message.down = False

    def __del__(self):
        self.unlink(kill=True)

    def unlink(self, kill=True):
        self.__mainDiv.unlink(kill)
        self.__mainDiv = None
        super(ScreenMessenger, self).unlink(kill)

    def isFinished(self):
        return self.__finished

    def getBackgroundColor(self):
        return self.__bgColor.fillcolor

    def getFontColor(self):
        return self.__headline.color

    def getFontSize(self):
        return self.__messageText.fontsize

    def addSubscription(self, function):
        self.__continueButton.subscribe(Button.BUTTON_CLICKED, function)
