####################################################################################
#
# Tilemap Editor for MazeRunner Game
#
####################################################################################

from EdConstants import *

import os, sys, inspect
sys.path.insert(1, os.path.join(sys.path[0], GAME_DIR))

from libavg import avg, app, player
import tkSimpleDialog
import Util

####################################################################################

class ScalableDivNode(avg.DivNode):
    def __init__(self, **kwargs):
        parent = None
        if 'parent' in kwargs:
            parent = kwargs.pop('parent', None)

        super(ScalableDivNode, self).__init__(**kwargs)
        if parent:
            self.registerInstance(self, parent)

        self.__scale = 1.0

    def setScale(self, scale):
        numChildren = self.getNumChildren()
        for i in xrange(numChildren):
            child = self.getChild(i)
            if isinstance(child, ScalableDivNode):
                child.scale = scale
            elif isinstance(child, avg.LineNode):
                child.pos1 = (child.pos1 / self.__scale) * scale
                child.pos2 = (child.pos2 / self.__scale) * scale
            else:
                child.pos = (child.pos / self.__scale) * scale
                child.size = (child.size / self.__scale) * scale
        self.__scale = scale

    def getScale(self):
        return self.__scale

    scale = property(getScale, setScale)

####################################################################################

def serializeVector(v):
    return {'x':v[0],'y':v[1]}

def deserializeVector(data):
    if isinstance(data, dict):
        return (data['x'],data['y'])
    else:
        return (data[0],data[1])

####################################################################################

# Shows a dialog asking the user to input a 2D vector and returns
# the vector as a tuple or None if input was invalid or False if cancelled.
def askfortuple(title="Tuple", text="Insert a tuple:", default=None):
    initial = (`default[0]` + ',' + `default[1]` if default else "")
    s = tkSimpleDialog.askstring(title, text, initialvalue=initial)
    if s is not None:
        sparts = s.split(',')
        if len(sparts) >= 2:
            return (int(sparts[0]), int(sparts[1]))
        else:
            return False
    else:
        return None

def askforstring(title="String", text="Enter a string:", default=''):
    return tkSimpleDialog.askstring(title, text, initialvalue=default)

####################################################################################

# Makes sure the given node is rendered above each other child of the node's parent
def bringNodeToTop(node):
    if node and node.parent:
        node.parent.reorderChild(node, node.parent.getNumChildren() - 1)
