####################################################################################
#
# Tilemap Editor for MazeRunner Game
#
####################################################################################

from EdConstants import *

import os, sys, inspect
sys.path.insert(1, os.path.join(sys.path[0], GAME_DIR))

from libavg import avg, app, player
from Game import Game
from Tile import Tile
from Direction import *
from event.MemoryEvent import *
from event.PipesEvent import *
from event.SpearTrap import *
from event.BridgeEvent import *
from event.ObstacleTile import *
from event.BrokenSwampRails import *
from event.TutorialEvent import *
from event.BrokenRail import *
from event.LeverRiddleEvent import *
from event.LockpickEvent import *
from event.Avalanche import *
from event.Portal import *
from event.Quicktime import *
from item.Petroleum import PetroleumTileFeature
from item.Brakefluid import BrakefluidTileFeature
from item.TurnBack import TurnBackRuneTileFeature
from item.Ping import PingRuneTileFeature
from EdTools import *
from EdWorld import *
import Util, EdUtil
import Tkinter as tk, tkFileDialog, tkMessageBox, tkSimpleDialog
import json
import math

####################################################################################

class Editor(app.MainDiv):
    def onInit(self):
        tkRoot = tk.Tk()
        tkRoot.withdraw()

        player.drawHelpers = False
        player.subscribe(player.KEY_UP, self.__onKeyUp)

        tileset = 'mine'
        self.__world = World(self, Game.tileRes, (Game.worldX,Game.worldY), tileset, self.__onTileClicked)

        self.__spawn = Placeable("spawn", ASSET_DIR + "/player.png", self.__world)
        self.__objective = Placeable("objective", ASSET_DIR + "/chest_loot.png", self.__world)

        self.__world.addObject(self.__spawn)
        self.__world.addObject(self.__objective)

        self.__world.subscribe(avg.Node.MOUSE_WHEEL, self.__onWorldMouseWheel)
        self.__world.subscribe(avg.Node.CURSOR_DOWN, self.__onWorldCursorDown)
        self.subscribe(avg.Node.CURSOR_UP, self.__onWorldCursorUp)
        self.__world.subscribe(avg.Node.CURSOR_MOTION, self.__onWorldCursorMotion)
        self.subscribe(avg.Node.CURSOR_MOTION, self.__onCursorMotion)

        self.__viewPanning = False # View is currently being moved
        self.__viewPanStartPos = None
        self.__viewPanStartMousePos = None

        # UI
        self.__menu = avg.DivNode(parent=self)
        self.__menuBackground = avg.RectNode(pos=(0,0), size=(MENU_WIDTH, RESOLUTION[1]), fillcolor="222222", fillopacity=0.85, parent=self.__menu)

        self.__brandNode = avg.WordsNode(pos=(10,10), text="MazeRun Editor", fontsize=20, color="888888", font="Arial", variant="Bold", parent=self.__menu)

        self.__toolMenu = avg.DivNode(pos=(10,330), parent=self.__menu)

        self.__toolStatusNode = avg.WordsNode(pos=(10,RESOLUTION[1]-80), text="", fontsize=13, color="448822", parent=self.__menu)
        self.__worldStatusNode = avg.WordsNode(pos=(10,RESOLUTION[1]-60), text="", fontsize=13, parent=self.__menu)
        self.__hoverStatusNode = avg.WordsNode(pos=(10,RESOLUTION[1]-45), text="", fontsize=13, parent=self.__menu)
        self.__zoomStatusNode = avg.WordsNode(pos=(10,RESOLUTION[1]-30), text="", fontsize=13, parent=self.__menu)

        self.__resizeWorld(self.__world.getWorldSize())

        # Tools:
        self.__tools = {}
        self.__toolKeys = {}
        self.addTool('1', PathTool(self.__world))
        self.addTool('2', PathBrushTool(self.__world))
        self.addTool('3', PlaceTool(self.__world, self.__toolMenu))

        self.__currentTool = PathBrushTool.NAME

        placeTool = self.__tools[PlaceTool.NAME]
        placeTool.registerObject('spawn', 'Spawn')
        placeTool.registerObject('objective', 'Objective')

        self.registerEventPlaceable(MemoryEvent.EVENT_NAME, "Memory Event", ASSET_DIR + "/door.png")
        self.registerEventPlaceable(PipesEvent.EVENT_NAME, "Pipes Event", ASSET_DIR + "/pipe_broken.png")
        self.registerEventPlaceable(LeverRiddleEvent.EVENT_NAME, "Lever Riddle", ASSET_DIR + "/leverriddle-wall.png")
        self.registerEventPlaceable(SpearTrap.FEATURE_NAME, "Spear trap", ASSET_DIR + "/spear-trap/spear-trap-active.png")
        self.registerEventPlaceable(SpearTrapLever.FEATURE_NAME, "Spear trap lever", ASSET_DIR + "/spear-trap/lever-active.png", True)
        self.registerEventPlaceable(BridgeEvent.EVENT_NAME, "Bridge event", ASSET_DIR + "/bridge.png")
        self.registerEventPlaceable(ObstacleTile.FEATURE_NAME, "Obstacle", ASSET_DIR + "/{tileset}/obstacle_large.png")
        self.registerEventPlaceable(BrokenSwampRails.EVENT_NAME, "Broken swamp", ASSET_DIR + "/{tileset}/ground.png")
        self.registerEventPlaceable(BrokenRail.EVENT_NAME, "Broken rail", ASSET_DIR + "/{tileset}/path_straight_0_broken.png")
        self.registerEventPlaceable(PetroleumTileFeature.FEATURE_NAME, "Petroleum", ASSET_DIR + "/items/petroleum.png")
        self.registerEventPlaceable(BrakefluidTileFeature.FEATURE_NAME, "Brakefluid", ASSET_DIR + "/items/brakefluid.png")
        self.registerEventPlaceable(TurnBackRuneTileFeature.FEATURE_NAME, "TurnBack", ASSET_DIR + "/items/turn_back.png")
        self.registerEventPlaceable(PingRuneTileFeature.FEATURE_NAME, "Ping", ASSET_DIR + "/items/ping.png")
        self.registerEventPlaceable(TutorialEvent.EVENT_NAME, "Tutorial", ASSET_DIR + "/mine/event.png")
        self.registerEventPlaceable(LockpickEvent.EVENT_NAME, "Lockpick", ASSET_DIR + "/lockpick/door_closed.png")
        self.registerEventPlaceable(Avalanche.EVENT_NAME, "Avalanche", ASSET_DIR + "/avalanche.png")
        self.registerEventPlaceable(Portal.FEATURE_NAME, "Portals", ASSET_DIR + "/portal.png")
        self.registerEventPlaceable(QuicktimeEvent.EVENT_NAME, "Quicktime", ASSET_DIR + "/quicktime-helper.png")

        self.__tools[PlaceTool.NAME].setPlaceable('spawn')

        # Help text
        helpTxt = "[Middle Mouse Btn] - Pan view<br />"
        helpTxt += "[Scroll wheel] - Zoom<br />"
        helpTxt += "[Left click] - Place<br />"
        helpTxt += "[Right click] - Remove<br />"
        helpTxt += "[G] - Toggle Grid<br />"
        helpTxt += "[B] - Toggle Background<br />"
        helpTxt += "[I] - Zoom to In-Game scale<br />"
        helpTxt += "[F] - Zoom to Fit<br />"
        helpTxt += "[C] - Clear world<br />"
        helpTxt += "[P] - Change world size...<br />"
        helpTxt += "[T] - Change BG texture size...<br />"
        helpTxt += "[Y] - Change tileset...<br />"
        helpTxt += "[S] - Save world as...<br />"
        helpTxt += "[L] - Load world...<br />"
        helpTxt += "<br />"

        for key in sorted(self.__toolKeys):
            tool = self.__tools[self.__toolKeys[key]]
            helpTxt += "[" + key + "] - " + tool.getDescription() + "<br />"

        self.__helpNode = avg.WordsNode(pos=(10,50), text=helpTxt, fontsize=13, parent=self.__menu)


        self.__fitZoom = 0.17
        self.__world.scale = self.__fitZoom
        self.__world.pos = avg.Point2D(RESOLUTION) * 0.5

        self.__clearWorld()

    def registerEventPlaceable(self, name, description, href, hidden=False):
        self.__world.addObject(Placeable(name, href, self.__world))
        self.__tools[PlaceTool.NAME].registerObject(name, description, True, hidden)

    def __updateWorldStatus(self):
        worldSz = self.__world.getWorldSize()
        self.__worldStatusNode.text = "World: " + `worldSz[0]` + "x" + `worldSz[1]`

    def onFrame(self):
        self.__zoomStatusNode.text = "zoom=" + `round(self.__world.scale, 2)`
        self.__toolStatusNode.text = "Tool: " + self.__tools[self.__currentTool].getDescription()
        pass

    #-------------------------------------------------------------------------------------------

    def __saveWorld(self, filename):
        world = self.__world

        pathStart = self.__spawn.getPos()
        if world.tileExists(pathStart):
            spawnTile = world.getTile(pathStart)
            if not spawnTile or spawnTile.texture == 'ground':
                print "WARNING: Invalid spawn point!"
        else:
            worldSz = world.getWorldSize()
            for y in xrange(worldSz[0]):
                for x in xrange(worldSz[1]):
                    tile = world.getTile((x,y))
                    if tile and tile.texture != 'ground':
                        pathStart = (x,y)
                        break
                if pathStart:
                    break

        if not pathStart:
            print "Cannot save world: There must be at least one path tile"
            return

        objectiveTile = world.getTile(self.__objective.pos)
        if not objectiveTile or objectiveTile.texture == 'ground':
            print "WARNING: Invalid objective position!"

        data = {
            'spawn': self.__spawn.serialize(),
            'objective': self.__objective.serialize(),
            'tileset': self.__world.tileset,
            'world': self.__world.serialize(pathStart)
        }

        with open(filename, 'w') as fo:
            fo.write(json.dumps(data, separators=[',',':']))
            print "Saved world as '" + filename + "'"

    #-------------------------------------------------------------------------------------------

    def __openWorld(self, filename):
        data = None
        with open(filename, 'r', 0) as fo:
            data = json.loads(fo.read())

        if not data:
            print "Failed load world '" + filename + "': Empty or not readable"
            return

        self.__clearWorld()

        if 'tileset' in data:
            self.__world.tileset = data['tileset']

        if 'world' in data:
            self.__world.deserialize(data['world'])
        else:
            print "Warning: World data missing in loaded world file!"

        if 'spawn' in data:
            self.__spawn.deserialize(data['spawn'])
            if not self.__world.tileExists(self.__spawn.pos):
                print "Warning: Invalid spawn point in loaded world file!"
        else:
            print "Warning: Spawn point missing in loaded world file!"

        if 'objective' in data:
            self.__objective.deserialize(data['objective'])
            if not self.__world.tileExists(self.__objective.pos):
                print "Warning: Invalid objective position in loaded world file!"
        else:
            print "Warning: Objective position missing in loaded world file!"

        print "World '" + filename + "' loaded"

    #-------------------------------------------------------------------------------------------

    # screenPos - Point2D
    def screenToWorldPos(self, screenPos):
        return self.__world.getWorldPixelSize() * 0.5 + (screenPos - self.__world.pos) / self.__world.scale

    def getTileAtScreenPos(self, screenPos):
        return self.__world.getTileAtPixelPos(self.screenToWorldPos(screenPos))

    def __clearWorld(self):
        self.__spawn.pos = (-1,-1)
        self.__objective.pos = (-1,-1)
        self.__world.clear()

    def __onWorldResizing(self, newsz):
        # Update spawn:
        if self.__spawn.pos < newsz:
            self.__spawn.pos = self.__spawn.pos # Update world pos
            self.__spawn.bringToTop()
        else:
            self.__spawnPos = (-1,-1) # invalid

        # Update objective:
        if self.__objective.pos < newsz:
            self.__objective.pos = self.__objective.pos
            self.__objective.bringToTop()
        else:
            self.__objectivePos = (-1,-1)

    def __resizeWorld(self, newsz):
        print "Resizing world to " + `newsz`
        self.__world.setWorldSize(newsz, self.__onWorldResizing)
        self.__updateWorldStatus()

    #-------------------------------------------------------------------------------------------

    def addTool(self, key, tool):
        name = tool.getName()
        if not name in self.__tools:
            self.__tools[name] = tool
            self.__toolKeys[key] = name

    def selectTool(self, name):
        if self.__currentTool != name:
            tool = self.__tools[self.__currentTool]
            if tool:
                tool.disable()

            self.__currentTool = name
            tool = self.__tools[name]
            tool.enable()

    #-------------------------------------------------------------------------------------------

    def __onKeyUp(self, event):
        for key in self.__toolKeys:
            if event.keyname == key:
                self.selectTool(self.__toolKeys[key])

        if event.keyname == 'G':
            self.__world.showGrid(not self.__world.isGridShown())
        elif event.keyname == 'B':
            self.__world.showBackground(not self.__world.isBackgroundShown())
        elif event.keyname == 'C':
            if tkMessageBox.askyesno("Clear world", "Do you really want to clear the world?\nUnsaved changes will be lost forever."):
                self.__clearWorld()
        elif event.keyname == 'I':
            self.__world.scale = 1.0
        elif event.keyname == 'F':
            self.__world.scale = self.__fitZoom
            self.__world.pos = (730,380)
        elif event.keyname == 'S':
            f = tkFileDialog.asksaveasfilename()
            if f:
                self.__saveWorld(f)
        elif event.keyname == 'L':
            f = tkFileDialog.askopenfilename()
            if f:
                self.__openWorld(f)
        elif event.keyname == 'P':
            newsz = askfortuple("New world size", "Enter new comma-separated world size in tiles, e.g. '42,30':")
            if newsz:
                self.__resizeWorld(newsz)
            elif newsz is None:
                tkMessageBox.showerror("Invalid input", "Invalid world-size entered")
        elif event.keyname == 'T':
            sz = self.__world.getBackgroundTextureSize()
            newsz = askfortuple("Change background texture size", "Enter a new comma-separated background texture size in pixels, e.g. 400,400:", sz)
            if newsz:
                self.__world.setBackgroundTextureSize(newsz)
            elif newsz is None:
                tkMessageBox.showerror("Invalid input", "Invalid background texture size entered")
        elif event.keyname == 'Y':
            newTileset = askforstring("Set tileset", "Enter the new tileset name:", self.__world.tileset)
            if newTileset:
                self.__world.tileset = newTileset
            elif newTileset == '':
                tkMessageBox.showerror("Invalid input", "Invalid tileset entered")

    def __onWorldMouseWheel(self, event):
        dz = event.motion.y * 0.03
        prevcursor = self.screenToWorldPos(event.pos)
        if self.__world.scale + dz <= 0.14:
            self.__world.scale = 0.14
        else:
            self.__world.scale += dz
        self.__world.pos += (self.screenToWorldPos(event.pos) - prevcursor) * self.__world.scale

    def __onWorldCursorDown(self, event):
        if isinstance(event, avg.MouseEvent):
            self.__viewPanning = event.middlebuttonstate
            if self.__viewPanning:
                self.__viewPanStartPos = self.__world.pos
                self.__viewPanStartMousePos = event.pos

    def __onWorldCursorUp(self, event):
        if isinstance(event, avg.MouseEvent):
            self.__viewPanning = event.middlebuttonstate

    def __onWorldCursorMotion(self, event):
        if self.__viewPanning:
            self.__world.pos = self.__viewPanStartPos + (event.pos - self.__viewPanStartMousePos)

        tile = self.getTileAtScreenPos(event.pos)
        if not tile:
            return

        event.tilePos = self.__world.getTilePos(tile)
        self.__tools[self.__currentTool].onWorldCursorMotion(event)

    def __onCursorMotion(self, event):
        self.__hoverStatusNode.text = "Hovered: "
        hoveredTile = self.getTileAtScreenPos(event.pos)
        if hoveredTile:
            self.__hoverStatusNode.text += "Tile@" + `self.__world.getTilePos(hoveredTile)`

            event.tilePos = self.__world.getTilePos(hoveredTile)
            self.__tools[self.__currentTool].onCursorMotion(event)
        else:
            self.__hoverStatusNode.text += "None"

    def __onTileClicked(self, event, tile):
        event.tilePos = self.__world.getTilePos(tile)
        event.lmouse = (event.button == 1)
        event.rmouse = (event.button == 2)
        tool = self.__tools[self.__currentTool]
        res = tool.onTileClicked(event, tile)
        if event.lmouse and self.__currentTool == PlaceTool.NAME and res is True:
            if tool.getPlaceable() == SpearTrap.FEATURE_NAME:
                tool.setPlaceable(SpearTrapLever.FEATURE_NAME)
            elif tool.getPlaceable() == SpearTrapLever.FEATURE_NAME:
                tool.setPlaceable(SpearTrap.FEATURE_NAME)

if __name__ == '__main__':
    windowSz = `RESOLUTION[0]` + 'x' + `RESOLUTION[1]`
    app.App().run(Editor(), app_resolution=windowSz, app_window_size=windowSz, app_fullscreen="False")
